"""
业务能力层集成测试
测试业务能力层各组件间的协同工作以及与其他层的集成
"""

import pytest
import asyncio
from datetime import datetime, timedelta
from unittest.mock import Mock, AsyncMock, patch
from typing import List, Dict, Any

from layers.business.teams.collaboration_manager import (
    CollaborationManager, CollaborationPattern, CollaborationRole, TeamMember
)
from layers.business.teams.team_engine import TeamManager
from layers.business.workflows.workflow_engine import (
    WorkflowEngine, WorkflowDefinition, WorkflowStep, WorkflowStepType
)
from layers.business.project import ProjectManager
from layers.business.communication_manager import BusinessCommunicationManager

from layers.framework.abstractions.agent import UniversalAgent, AgentCapability
from layers.framework.abstractions.task import UniversalTask, TaskType, TaskPriority
from layers.framework.abstractions.context import UniversalContext
from layers.framework.abstractions.result import UniversalResult, ResultStatus


class TestBusinessLayerIntegration:
    """业务能力层内部集成测试"""
    
    @pytest.fixture
    def business_components(self):
        """创建业务能力层所有组件"""
        return {
            "collaboration_manager": CollaborationManager(),
            "team_manager": TeamManager(),
            "workflow_engine": WorkflowEngine(),
            "project_manager": ProjectManager(),
            "communication_manager": BusinessCommunicationManager()
        }
    
    @pytest.fixture
    def mock_agents(self):
        """创建完整的模拟Agent团队"""
        agents = []
        roles_and_capabilities = [
            ("project_manager", [AgentCapability.PROJECT_MANAGEMENT, AgentCapability.REASONING]),
            ("architect", [AgentCapability.ARCHITECTURE_DESIGN, AgentCapability.REASONING]),
            ("developer_1", [AgentCapability.CODE_GENERATION, AgentCapability.DEBUGGING]),
            ("developer_2", [AgentCapability.CODE_GENERATION, AgentCapability.TESTING]),
            ("tester", [AgentCapability.TESTING, AgentCapability.QUALITY_ASSURANCE]),
            ("designer", [AgentCapability.UI_UX_DESIGN, AgentCapability.VISUALIZATION]),
            ("analyst", [AgentCapability.DATA_ANALYSIS, AgentCapability.REASONING])
        ]
        
        for role, capabilities in roles_and_capabilities:
            agent = Mock(spec=UniversalAgent)
            agent.agent_id = f"{role}_agent"
            agent.name = f"{role.replace('_', ' ').title()} Agent"
            agent.role = role
            agent.capabilities = capabilities
            agent.performance_score = 0.8
            agent.availability = True
            
            async def mock_execute(task, agent_role=role):
                await asyncio.sleep(0.1)  # 模拟处理时间
                
                # 根据角色返回不同类型的结果
                if "manager" in agent_role:
                    result_data = {"plan": "Project plan created", "timeline": "2 weeks"}
                elif "developer" in agent_role:
                    result_data = {"code": "Implementation completed", "tests": "Unit tests passed"}
                elif "tester" in agent_role:
                    result_data = {"test_results": "All tests passed", "coverage": "95%"}
                elif "designer" in agent_role:
                    result_data = {"design": "UI mockups created", "prototype": "Interactive prototype"}
                else:
                    result_data = {"analysis": "Analysis completed", "insights": ["insight1", "insight2"]}
                
                return UniversalResult(
                    task_id=task.task_id,
                    status=ResultStatus.SUCCESS,
                    data=result_data,
                    metadata={"agent_id": agent.agent_id, "role": agent_role}
                )
            
            agent.execute = mock_execute
            agents.append(agent)
        
        return agents

    @pytest.mark.asyncio
    async def test_team_workflow_integration(self, business_components, mock_agents):
        """测试团队管理与工作流引擎的集成"""
        team_manager = business_components["team_manager"]
        workflow_engine = business_components["workflow_engine"]
        collaboration_manager = business_components["collaboration_manager"]
        
        # 1. 创建开发团队
        team_id = "dev_team_integration"
        dev_agents = mock_agents[:5]  # PM, 架构师, 2个开发者, 测试者
        
        await team_manager.create_team(
            team_id=team_id,
            team_name="Development Team",
            agents=dev_agents,
            team_config={
                "collaboration_pattern": CollaborationPattern.HIERARCHICAL,
                "max_size": 10
            }
        )
        
        # 2. 注册Agent到工作流引擎
        for agent in dev_agents:
            await workflow_engine.register_agent(agent)
        
        # 3. 定义软件开发工作流
        dev_workflow = WorkflowDefinition(
            workflow_id="software_dev_workflow",
            name="Software Development Workflow",
            steps=[
                WorkflowStep(
                    step_id="planning",
                    name="Project Planning",
                    step_type=WorkflowStepType.AGENT_TASK,
                    config={"agent_id": "project_manager_agent"},
                    timeout=60
                ),
                WorkflowStep(
                    step_id="architecture",
                    name="Architecture Design",
                    step_type=WorkflowStepType.AGENT_TASK,
                    config={"agent_id": "architect_agent"},
                    dependencies=["planning"],
                    timeout=60
                ),
                WorkflowStep(
                    step_id="development",
                    name="Parallel Development",
                    step_type=WorkflowStepType.PARALLEL,
                    config={
                        "parallel_steps": [
                            {
                                "step_id": "dev_1",
                                "step_type": "agent_task",
                                "config": {"agent_id": "developer_1_agent"}
                            },
                            {
                                "step_id": "dev_2",
                                "step_type": "agent_task", 
                                "config": {"agent_id": "developer_2_agent"}
                            }
                        ]
                    },
                    dependencies=["architecture"],
                    timeout=120
                ),
                WorkflowStep(
                    step_id="testing",
                    name="Quality Assurance",
                    step_type=WorkflowStepType.AGENT_TASK,
                    config={"agent_id": "tester_agent"},
                    dependencies=["development"],
                    timeout=60
                )
            ],
            timeout=300
        )
        
        # 4. 注册工作流
        await workflow_engine.register_workflow(dev_workflow)
        
        # 5. 创建项目任务
        project_task = UniversalTask(
            task_id="integration_project",
            task_type=TaskType.PROJECT,
            description="Develop new feature with team workflow integration",
            priority=TaskPriority.HIGH,
            context=UniversalContext(
                session_id="integration_session",
                data={
                    "feature_requirements": "User authentication system",
                    "deadline": "2024-02-01",
                    "team_id": team_id
                }
            )
        )
        
        # 6. 执行工作流
        execution_id = await workflow_engine.start_workflow("software_dev_workflow", project_task)
        execution = await workflow_engine.wait_for_completion(execution_id, timeout=30)
        
        # 7. 验证集成结果
        assert execution.status.value == "completed"
        assert len(execution.step_executions) == 4
        
        # 验证每个步骤都成功完成
        for step_id in ["planning", "architecture", "development", "testing"]:
            if step_id in execution.step_executions:
                step_execution = execution.step_executions[step_id]
                assert step_execution.status.value == "completed"
        
        # 8. 验证团队性能指标
        team_performance = await team_manager.get_team_performance(team_id)
        assert team_performance["overall_score"] > 0.5

    @pytest.mark.asyncio
    async def test_collaboration_workflow_integration(self, business_components, mock_agents):
        """测试协作管理与工作流的深度集成"""
        collaboration_manager = business_components["collaboration_manager"]
        workflow_engine = business_components["workflow_engine"]
        
        # 1. 创建协作团队
        team_id = "collaboration_integration_team"
        collab_agents = mock_agents[2:6]  # 开发者、测试者、设计师、分析师
        
        members = [
            TeamMember(agent=collab_agents[0], role=CollaborationRole.LEADER,
                      capabilities=collab_agents[0].capabilities),
            TeamMember(agent=collab_agents[1], role=CollaborationRole.EXPERT,
                      capabilities=collab_agents[1].capabilities),
            TeamMember(agent=collab_agents[2], role=CollaborationRole.CONTRIBUTOR,
                      capabilities=collab_agents[2].capabilities),
            TeamMember(agent=collab_agents[3], role=CollaborationRole.REVIEWER,
                      capabilities=collab_agents[3].capabilities)
        ]
        
        await collaboration_manager.create_team(team_id, members)
        
        # 2. 注册Agent到工作流引擎
        for agent in collab_agents:
            await workflow_engine.register_agent(agent)
        
        # 3. 定义协作工作流（包含多种协作模式）
        collaboration_workflow = WorkflowDefinition(
            workflow_id="collaboration_workflow",
            name="Multi-Pattern Collaboration Workflow",
            steps=[
                WorkflowStep(
                    step_id="brainstorm",
                    name="Parallel Brainstorming",
                    step_type=WorkflowStepType.PARALLEL,
                    config={
                        "collaboration_pattern": "parallel",
                        "team_id": team_id,
                        "parallel_steps": [
                            {"step_id": "idea_1", "step_type": "agent_task", 
                             "config": {"agent_id": "developer_1_agent"}},
                            {"step_id": "idea_2", "step_type": "agent_task",
                             "config": {"agent_id": "designer_agent"}},
                            {"step_id": "idea_3", "step_type": "agent_task",
                             "config": {"agent_id": "analyst_agent"}}
                        ]
                    },
                    timeout=90
                ),
                WorkflowStep(
                    step_id="expert_review",
                    name="Expert Consultation",
                    step_type=WorkflowStepType.AGENT_TASK,
                    config={
                        "collaboration_pattern": "expert_consultation",
                        "team_id": team_id,
                        "agent_id": "tester_agent"
                    },
                    dependencies=["brainstorm"],
                    timeout=60
                ),
                WorkflowStep(
                    step_id="consensus_decision",
                    name="Consensus Building",
                    step_type=WorkflowStepType.CONDITION,
                    config={
                        "collaboration_pattern": "consensus",
                        "team_id": team_id,
                        "condition": "consensus_score >= 0.8"
                    },
                    dependencies=["expert_review"],
                    timeout=60
                )
            ],
            timeout=300
        )
        
        await workflow_engine.register_workflow(collaboration_workflow)
        
        # 4. 执行协作工作流
        collab_task = UniversalTask(
            task_id="collaboration_integration_task",
            task_type=TaskType.COLLABORATION,
            description="Multi-pattern collaboration integration test",
            priority=TaskPriority.MEDIUM
        )
        
        execution_id = await workflow_engine.start_workflow("collaboration_workflow", collab_task)
        execution = await workflow_engine.wait_for_completion(execution_id, timeout=30)
        
        # 5. 验证协作工作流结果
        assert execution.status.value == "completed"
        
        # 验证协作历史记录
        collab_history = collaboration_manager.collaboration_history
        assert len(collab_history) > 0

    @pytest.mark.asyncio
    async def test_project_team_workflow_integration(self, business_components, mock_agents):
        """测试项目管理、团队管理和工作流的三方集成"""
        project_manager = business_components["project_manager"]
        team_manager = business_components["team_manager"]
        workflow_engine = business_components["workflow_engine"]
        
        # 1. 创建项目
        project_config = {
            "project_id": "integration_project_001",
            "name": "Full Stack Integration Project",
            "description": "Complete integration test project",
            "timeline": {
                "start_date": datetime.now(),
                "end_date": datetime.now() + timedelta(days=30)
            },
            "requirements": {
                "features": ["authentication", "dashboard", "reporting"],
                "quality_standards": {"test_coverage": 90, "performance": "sub_2s"}
            }
        }
        
        project = await project_manager.create_project(project_config)
        
        # 2. 为项目分配团队
        project_team_id = f"team_{project_config['project_id']}"
        project_agents = mock_agents  # 使用全部Agent
        
        await team_manager.create_team(
            team_id=project_team_id,
            team_name=f"Team for {project_config['name']}",
            agents=project_agents,
            team_config={
                "project_id": project_config["project_id"],
                "collaboration_pattern": CollaborationPattern.HIERARCHICAL
            }
        )
        
        # 3. 将团队分配给项目
        await project_manager.assign_team(project_config["project_id"], project_team_id)
        
        # 4. 注册项目Agent到工作流引擎
        for agent in project_agents:
            await workflow_engine.register_agent(agent)
        
        # 5. 定义项目工作流
        project_workflow = WorkflowDefinition(
            workflow_id="full_project_workflow",
            name="Complete Project Workflow",
            steps=[
                # 项目启动阶段
                WorkflowStep(
                    step_id="project_kickoff",
                    name="Project Kickoff",
                    step_type=WorkflowStepType.AGENT_TASK,
                    config={"agent_id": "project_manager_agent"},
                    timeout=30
                ),
                # 需求分析阶段
                WorkflowStep(
                    step_id="requirements_analysis",
                    name="Requirements Analysis",
                    step_type=WorkflowStepType.PARALLEL,
                    config={
                        "parallel_steps": [
                            {"step_id": "business_analysis", "step_type": "agent_task",
                             "config": {"agent_id": "analyst_agent"}},
                            {"step_id": "technical_analysis", "step_type": "agent_task",
                             "config": {"agent_id": "architect_agent"}}
                        ]
                    },
                    dependencies=["project_kickoff"],
                    timeout=60
                ),
                # 设计阶段
                WorkflowStep(
                    step_id="design_phase",
                    name="Design Phase",
                    step_type=WorkflowStepType.PARALLEL,
                    config={
                        "parallel_steps": [
                            {"step_id": "ui_design", "step_type": "agent_task",
                             "config": {"agent_id": "designer_agent"}},
                            {"step_id": "architecture_design", "step_type": "agent_task",
                             "config": {"agent_id": "architect_agent"}}
                        ]
                    },
                    dependencies=["requirements_analysis"],
                    timeout=90
                ),
                # 开发阶段
                WorkflowStep(
                    step_id="development_phase",
                    name="Development Phase",
                    step_type=WorkflowStepType.PARALLEL,
                    config={
                        "parallel_steps": [
                            {"step_id": "frontend_dev", "step_type": "agent_task",
                             "config": {"agent_id": "developer_1_agent"}},
                            {"step_id": "backend_dev", "step_type": "agent_task",
                             "config": {"agent_id": "developer_2_agent"}}
                        ]
                    },
                    dependencies=["design_phase"],
                    timeout=120
                ),
                # 测试阶段
                WorkflowStep(
                    step_id="testing_phase",
                    name="Testing Phase",
                    step_type=WorkflowStepType.AGENT_TASK,
                    config={"agent_id": "tester_agent"},
                    dependencies=["development_phase"],
                    timeout=60
                ),
                # 项目收尾
                WorkflowStep(
                    step_id="project_closure",
                    name="Project Closure",
                    step_type=WorkflowStepType.AGENT_TASK,
                    config={"agent_id": "project_manager_agent"},
                    dependencies=["testing_phase"],
                    timeout=30
                )
            ],
            timeout=600
        )
        
        await workflow_engine.register_workflow(project_workflow)
        
        # 6. 启动项目工作流
        project_task = UniversalTask(
            task_id="full_project_task",
            task_type=TaskType.PROJECT,
            description="Execute complete project workflow",
            priority=TaskPriority.HIGH,
            context=UniversalContext(
                session_id="project_session",
                data={
                    "project_id": project_config["project_id"],
                    "team_id": project_team_id,
                    "requirements": project_config["requirements"]
                }
            )
        )
        
        execution_id = await workflow_engine.start_workflow("full_project_workflow", project_task)
        execution = await workflow_engine.wait_for_completion(execution_id, timeout=60)
        
        # 7. 验证项目执行结果
        assert execution.status.value == "completed"
        assert len(execution.step_executions) == 6
        
        # 验证项目状态更新
        project_status = await project_manager.get_project_status(project_config["project_id"])
        assert project_status["status"] in ["in_progress", "completed"]
        
        # 验证团队性能
        team_performance = await team_manager.get_team_performance(project_team_id)
        assert team_performance["overall_score"] > 0.6
        
        # 验证工作流执行质量
        workflow_stats = workflow_engine.get_workflow_statistics("full_project_workflow")
        assert workflow_stats["success_rate"] > 0.8


class TestCrossLayerIntegration:
    """跨层集成测试"""
    
    @pytest.fixture
    def business_components(self):
        return {
            "collaboration_manager": CollaborationManager(),
            "team_manager": TeamManager(),
            "workflow_engine": WorkflowEngine(),
            "communication_manager": BusinessCommunicationManager()
        }
    
    @pytest.fixture
    def mock_framework_layer(self):
        """模拟框架抽象层"""
        framework_mock = Mock()
        
        # 模拟Agent工厂
        framework_mock.agent_factory.create_agent = AsyncMock()
        framework_mock.agent_factory.get_available_agents = AsyncMock(return_value=[])
        
        # 模拟上下文管理
        framework_mock.context_manager.create_context = AsyncMock()
        framework_mock.context_manager.update_context = AsyncMock()
        
        return framework_mock
    
    @pytest.fixture
    def mock_cognitive_layer(self):
        """模拟认知架构层"""
        cognitive_mock = Mock()
        
        # 模拟认知Agent
        cognitive_mock.cognitive_agent.process_task = AsyncMock()
        cognitive_mock.memory_system.store_experience = AsyncMock()
        cognitive_mock.learning_module.update_knowledge = AsyncMock()
        
        return cognitive_mock

    @pytest.mark.asyncio
    async def test_business_framework_integration(self, business_components, mock_framework_layer, mock_agents):
        """测试业务能力层与框架抽象层的集成"""
        communication_manager = business_components["communication_manager"]
        team_manager = business_components["team_manager"]
        
        # 1. 设置跨层通信
        await communication_manager.register_layer_connection("framework", mock_framework_layer)
        
        # 2. 创建团队
        team_id = "framework_integration_team"
        await team_manager.create_team(
            team_id=team_id,
            team_name="Framework Integration Team",
            agents=mock_agents[:3]
        )
        
        # 3. 请求框架层创建专门的Agent
        agent_request = {
            "agent_type": "specialized_analyst",
            "capabilities": [AgentCapability.DATA_ANALYSIS, AgentCapability.REASONING],
            "performance_requirements": {"min_score": 0.8}
        }
        
        # 模拟框架层响应
        specialized_agent = Mock(spec=UniversalAgent)
        specialized_agent.agent_id = "specialized_analyst_001"
        specialized_agent.capabilities = agent_request["capabilities"]
        mock_framework_layer.agent_factory.create_agent.return_value = specialized_agent
        
        # 4. 通过层间通信请求Agent
        response = await communication_manager.send_layer_request(
            target_layer="framework",
            request_type="create_agent",
            data=agent_request
        )
        
        # 5. 验证跨层通信
        mock_framework_layer.agent_factory.create_agent.assert_called_once()
        assert response is not None
        
        # 6. 将新Agent添加到团队
        await team_manager.add_team_member(
            team_id, specialized_agent, CollaborationRole.EXPERT
        )
        
        # 验证团队更新
        team = team_manager.teams[team_id]
        assert len(team["members"]) == 4

    @pytest.mark.asyncio
    async def test_business_cognitive_integration(self, business_components, mock_cognitive_layer, mock_agents):
        """测试业务能力层与认知架构层的集成"""
        communication_manager = business_components["communication_manager"]
        collaboration_manager = business_components["collaboration_manager"]
        
        # 1. 设置与认知层的连接
        await communication_manager.register_layer_connection("cognitive", mock_cognitive_layer)
        
        # 2. 创建协作团队
        team_id = "cognitive_integration_team"
        members = [
            TeamMember(agent=mock_agents[0], role=CollaborationRole.LEADER,
                      capabilities=mock_agents[0].capabilities),
            TeamMember(agent=mock_agents[1], role=CollaborationRole.EXPERT,
                      capabilities=mock_agents[1].capabilities)
        ]
        
        await collaboration_manager.create_team(team_id, members)
        
        # 3. 执行需要认知增强的协作任务
        cognitive_task = UniversalTask(
            task_id="cognitive_enhanced_task",
            task_type=TaskType.COMPLEX_REASONING,
            description="Complex problem requiring cognitive enhancement",
            priority=TaskPriority.HIGH,
            context=UniversalContext(
                session_id="cognitive_session",
                data={
                    "complexity_level": "high",
                    "requires_learning": True,
                    "memory_intensive": True
                }
            )
        )
        
        # 4. 模拟认知层处理
        mock_cognitive_layer.cognitive_agent.process_task.return_value = UniversalResult(
            task_id=cognitive_task.task_id,
            status=ResultStatus.SUCCESS,
            data={
                "cognitive_analysis": "Deep analysis completed",
                "insights": ["insight1", "insight2", "insight3"],
                "confidence": 0.92
            },
            metadata={"cognitive_processing": True, "learning_applied": True}
        )
        
        # 5. 执行协作（集成认知处理）
        result = await collaboration_manager.collaborate(
            team_id=team_id,
            task=cognitive_task,
            pattern=CollaborationPattern.EXPERT_CONSULTATION,
            config={"cognitive_enhancement": True}
        )
        
        # 6. 验证认知集成
        assert result.final_result.status == ResultStatus.SUCCESS
        
        # 验证认知层调用
        mock_cognitive_layer.cognitive_agent.process_task.assert_called()
        mock_cognitive_layer.memory_system.store_experience.assert_called()

    @pytest.mark.asyncio
    async def test_end_to_end_integration(self, business_components, mock_framework_layer, mock_cognitive_layer, mock_agents):
        """测试端到端集成"""
        # 获取所有业务组件
        team_manager = business_components["team_manager"]
        workflow_engine = business_components["workflow_engine"]
        communication_manager = business_components["communication_manager"]
        
        # 1. 设置所有层间连接
        await communication_manager.register_layer_connection("framework", mock_framework_layer)
        await communication_manager.register_layer_connection("cognitive", mock_cognitive_layer)
        
        # 2. 创建端到端项目
        e2e_team_id = "e2e_integration_team"
        await team_manager.create_team(
            team_id=e2e_team_id,
            team_name="End-to-End Integration Team",
            agents=mock_agents,
            team_config={
                "enable_cognitive_enhancement": True,
                "framework_integration": True
            }
        )
        
        # 3. 注册Agent到工作流引擎
        for agent in mock_agents:
            await workflow_engine.register_agent(agent)
        
        # 4. 定义端到端工作流
        e2e_workflow = WorkflowDefinition(
            workflow_id="e2e_integration_workflow",
            name="End-to-End Integration Workflow",
            steps=[
                # 请求框架层创建专门Agent
                WorkflowStep(
                    step_id="request_specialized_agent",
                    name="Request Specialized Agent",
                    step_type=WorkflowStepType.SCRIPT,
                    config={
                        "script": "request_framework_agent('data_scientist')",
                        "layer_integration": True
                    },
                    timeout=30
                ),
                # 认知增强的分析阶段
                WorkflowStep(
                    step_id="cognitive_analysis",
                    name="Cognitive Enhanced Analysis",
                    step_type=WorkflowStepType.AGENT_TASK,
                    config={
                        "agent_id": "analyst_agent",
                        "cognitive_enhancement": True
                    },
                    dependencies=["request_specialized_agent"],
                    timeout=60
                ),
                # 团队协作阶段
                WorkflowStep(
                    step_id="team_collaboration",
                    name="Team Collaboration",
                    step_type=WorkflowStepType.PARALLEL,
                    config={
                        "team_id": e2e_team_id,
                        "collaboration_pattern": "expert_consultation",
                        "parallel_steps": [
                            {"step_id": "dev_work", "step_type": "agent_task",
                             "config": {"agent_id": "developer_1_agent"}},
                            {"step_id": "design_work", "step_type": "agent_task",
                             "config": {"agent_id": "designer_agent"}}
                        ]
                    },
                    dependencies=["cognitive_analysis"],
                    timeout=90
                ),
                # 最终整合
                WorkflowStep(
                    step_id="final_integration",
                    name="Final Integration",
                    step_type=WorkflowStepType.AGENT_TASK,
                    config={
                        "agent_id": "project_manager_agent",
                        "integrate_all_results": True
                    },
                    dependencies=["team_collaboration"],
                    timeout=30
                )
            ],
            timeout=300
        )
        
        await workflow_engine.register_workflow(e2e_workflow)
        
        # 5. 执行端到端工作流
        e2e_task = UniversalTask(
            task_id="e2e_integration_task",
            task_type=TaskType.COMPLEX_PROJECT,
            description="Complete end-to-end integration test",
            priority=TaskPriority.CRITICAL,
            context=UniversalContext(
                session_id="e2e_session",
                data={
                    "integration_scope": "full_stack",
                    "quality_requirements": "enterprise_grade",
                    "team_id": e2e_team_id
                }
            )
        )
        
        # 模拟各层响应
        mock_framework_layer.agent_factory.create_agent.return_value = mock_agents[-1]
        mock_cognitive_layer.cognitive_agent.process_task.return_value = UniversalResult(
            task_id=e2e_task.task_id,
            status=ResultStatus.SUCCESS,
            data={"enhanced_analysis": "Complete cognitive analysis"},
            metadata={"cognitive_processing": True}
        )
        
        execution_id = await workflow_engine.start_workflow("e2e_integration_workflow", e2e_task)
        execution = await workflow_engine.wait_for_completion(execution_id, timeout=60)
        
        # 6. 验证端到端集成
        assert execution.status.value == "completed"
        
        # 验证各层都被调用
        mock_framework_layer.agent_factory.create_agent.assert_called()
        mock_cognitive_layer.cognitive_agent.process_task.assert_called()
        
        # 验证团队性能
        final_performance = await team_manager.get_team_performance(e2e_team_id)
        assert final_performance["overall_score"] > 0.7
        
        # 验证工作流完整性
        assert len(execution.step_executions) == 4
        for step_execution in execution.step_executions.values():
            assert step_execution.status.value in ["completed", "skipped"]


class TestBusinessLayerPerformance:
    """业务能力层性能测试"""
    
    @pytest.fixture
    def business_components(self):
        return {
            "collaboration_manager": CollaborationManager(),
            "team_manager": TeamManager(),
            "workflow_engine": WorkflowEngine()
        }

    @pytest.mark.asyncio
    async def test_high_concurrency_collaboration(self, business_components, mock_agents):
        """测试高并发协作性能"""
        collaboration_manager = business_components["collaboration_manager"]
        
        # 创建多个团队
        teams = []
        for i in range(5):
            team_id = f"concurrent_team_{i}"
            members = [
                TeamMember(agent=mock_agents[j % len(mock_agents)], 
                          role=CollaborationRole.CONTRIBUTOR,
                          capabilities=mock_agents[j % len(mock_agents)].capabilities)
                for j in range(3)
            ]
            await collaboration_manager.create_team(team_id, members)
            teams.append(team_id)
        
        # 并发执行协作任务
        tasks = []
        for i in range(20):  # 20个并发任务
            task = UniversalTask(
                task_id=f"concurrent_task_{i}",
                task_type=TaskType.ANALYSIS,
                description=f"Concurrent analysis task {i}"
            )
            
            team_id = teams[i % len(teams)]
            coroutine = collaboration_manager.collaborate(
                team_id=team_id,
                task=task,
                pattern=CollaborationPattern.PARALLEL
            )
            tasks.append(coroutine)
        
        # 等待所有任务完成
        start_time = datetime.now()
        results = await asyncio.gather(*tasks)
        end_time = datetime.now()
        
        # 验证性能
        execution_time = (end_time - start_time).total_seconds()
        assert execution_time < 10.0  # 应该在10秒内完成
        
        # 验证所有任务都成功
        success_count = sum(1 for result in results 
                           if result.final_result.status == ResultStatus.SUCCESS)
        assert success_count >= 18  # 至少90%成功率

    @pytest.mark.asyncio
    async def test_workflow_scalability(self, business_components, mock_agents):
        """测试工作流可扩展性"""
        workflow_engine = business_components["workflow_engine"]
        
        # 注册Agent
        for agent in mock_agents:
            await workflow_engine.register_agent(agent)
        
        # 创建可扩展工作流
        scalable_workflow = WorkflowDefinition(
            workflow_id="scalable_workflow",
            name="Scalable Workflow",
            steps=[
                WorkflowStep(
                    step_id=f"step_{i}",
                    name=f"Step {i}",
                    step_type=WorkflowStepType.AGENT_TASK,
                    config={"agent_id": mock_agents[i % len(mock_agents)].agent_id},
                    dependencies=[f"step_{i-1}"] if i > 0 else []
                )
                for i in range(10)  # 10步工作流
            ]
        )
        
        await workflow_engine.register_workflow(scalable_workflow)
        
        # 并发执行多个工作流实例
        execution_tasks = []
        for i in range(10):
            task = UniversalTask(
                task_id=f"scalable_task_{i}",
                task_type=TaskType.WORKFLOW,
                description=f"Scalable workflow execution {i}"
            )
            
            coroutine = workflow_engine.start_workflow("scalable_workflow", task)
            execution_tasks.append(coroutine)
        
        # 启动所有工作流
        execution_ids = await asyncio.gather(*execution_tasks)
        
        # 等待所有完成
        completion_tasks = [
            workflow_engine.wait_for_completion(execution_id, timeout=30)
            for execution_id in execution_ids
        ]
        
        executions = await asyncio.gather(*completion_tasks)
        
        # 验证可扩展性
        completed_count = sum(1 for execution in executions 
                             if execution.status.value == "completed")
        assert completed_count >= 8  # 至少80%成功完成

    @pytest.mark.asyncio
    async def test_memory_efficiency(self, business_components, mock_agents):
        """测试内存使用效率"""
        import psutil
        import os
        
        process = psutil.Process(os.getpid())
        initial_memory = process.memory_info().rss
        
        team_manager = business_components["team_manager"]
        
        # 创建大量团队和执行任务
        for i in range(50):
            team_id = f"memory_test_team_{i}"
            await team_manager.create_team(
                team_id=team_id,
                team_name=f"Memory Test Team {i}",
                agents=mock_agents[:3]
            )
            
            # 执行任务
            task = UniversalTask(
                task_id=f"memory_task_{i}",
                task_type=TaskType.ANALYSIS,
                description=f"Memory test task {i}"
            )
            
            await team_manager.execute_team_task(
                team_id, task, CollaborationPattern.SEQUENTIAL
            )
        
        # 检查内存使用
        final_memory = process.memory_info().rss
        memory_increase = final_memory - initial_memory
        
        # 内存增长应该在合理范围内（小于200MB）
        assert memory_increase < 200 * 1024 * 1024


if __name__ == "__main__":
    pytest.main([__file__, "-v", "--tb=short"]) 


"""
Universal Task Abstraction
通用任务抽象，定义框架无关的任务表示
"""

from typing import Dict, Any, List, Optional
from enum import Enum
from dataclasses import dataclass, field
from datetime import datetime
import uuid


class TaskPriority(Enum):
    """任务优先级"""
    LOW = "low"
    NORMAL = "normal"
    HIGH = "high"
    URGENT = "urgent"


class TaskStatus(Enum):
    """任务状态"""
    PENDING = "pending"
    RUNNING = "running"
    COMPLETED = "completed"
    FAILED = "failed"
    CANCELLED = "cancelled"


class TaskType(Enum):
    """任务类型"""
    CONVERSATION = "conversation"
    CODE_GENERATION = "code_generation"
    CODE_EXECUTION = "code_execution"
    FILE_OPERATION = "file_operation"
    WEB_SEARCH = "web_search"
    ANALYSIS = "analysis"
    PLANNING = "planning"
    TOOL_EXECUTION = "tool_execution"
    CUSTOM = "custom"
    
    # 业务任务类型
    CREATIVE = "creative"
    DECISION_MAKING = "decision_making"
    PROBLEM_SOLVING = "problem_solving"
    PROJECT = "project"
    COMPLEX_PROJECT = "complex_project"
    WORKFLOW = "workflow"
    COLLABORATION = "collaboration"
    COMPLEX_REASONING = "complex_reasoning"


@dataclass
class TaskRequirements:
    """任务需求"""
    capabilities: List[str] = field(default_factory=list)
    max_execution_time: Optional[int] = None  # seconds
    memory_limit: Optional[int] = None  # MB
    preferred_framework: Optional[str] = None
    fallback_frameworks: List[str] = field(default_factory=list)


@dataclass
class TaskMetadata:
    """任务元数据"""
    created_at: datetime = field(default_factory=datetime.now)
    started_at: Optional[datetime] = None
    completed_at: Optional[datetime] = None
    execution_time: Optional[float] = None
    retries: int = 0
    max_retries: int = 3


class UniversalTask:
    """
    通用任务类
    
    提供框架无关的任务表示
    """
    
    def __init__(self,
                 content: Optional[str] = None,
                 task_type: TaskType = TaskType.CONVERSATION,
                 priority: TaskPriority = TaskPriority.NORMAL,
                 requirements: Optional[TaskRequirements] = None,
                 context: Optional[Dict[str, Any]] = None,
                 task_id: Optional[str] = None,
                 description: Optional[str] = None):
        
        self.id = task_id or str(uuid.uuid4())
        self.task_id = self.id  # 添加task_id别名以保持兼容性
        # 支持content或description参数
        self.content = content or description or ""
        self.description = description or content or ""
        self.task_type = task_type
        self.priority = priority
        self.requirements = requirements or TaskRequirements()
        self.context = context or {}
        self.status = TaskStatus.PENDING
        self.metadata = TaskMetadata()
        
        # 存储任务结果和错误信息
        self.result: Optional['UniversalResult'] = None
        self.error: Optional[str] = None
    
    def start(self) -> None:
        """开始执行任务"""
        self.status = TaskStatus.RUNNING
        self.metadata.started_at = datetime.now()
    
    def complete(self, result: 'UniversalResult') -> None:
        """完成任务"""
        self.status = TaskStatus.COMPLETED
        self.result = result
        self.metadata.completed_at = datetime.now()
        
        if self.metadata.started_at:
            self.metadata.execution_time = (self.metadata.completed_at - self.metadata.started_at).total_seconds()
    
    def fail(self, error: str) -> None:
        """任务失败"""
        self.status = TaskStatus.FAILED
        self.error = error
        self.metadata.completed_at = datetime.now()
        
        if self.metadata.started_at:
            self.metadata.execution_time = (self.metadata.completed_at - self.metadata.started_at).total_seconds()
    
    def cancel(self) -> None:
        """取消任务"""
        self.status = TaskStatus.CANCELLED
        self.metadata.completed_at = datetime.now()
        
        if self.metadata.started_at:
            self.metadata.execution_time = (self.metadata.completed_at - self.metadata.started_at).total_seconds()
    
    def retry(self) -> bool:
        """重试任务"""
        if self.metadata.retries < self.metadata.max_retries:
            self.metadata.retries += 1
            self.status = TaskStatus.PENDING
            self.error = None
            self.result = None
            return True
        return False
    
    def add_context(self, key: str, value: Any) -> None:
        """添加上下文信息"""
        self.context[key] = value
    
    def get_context(self, key: str, default: Any = None) -> Any:
        """获取上下文信息"""
        return self.context.get(key, default)
    
    def has_context(self, key: str) -> bool:
        """检查是否有指定的上下文信息"""
        return key in self.context
    
    def to_dict(self) -> Dict[str, Any]:
        """转换为字典"""
        return {
            "id": self.id,
            "content": self.content,
            "task_type": self.task_type.value,
            "priority": self.priority.value,
            "status": self.status.value,
            "requirements": {
                "capabilities": self.requirements.capabilities,
                "max_execution_time": self.requirements.max_execution_time,
                "memory_limit": self.requirements.memory_limit,
                "preferred_framework": self.requirements.preferred_framework,
                "fallback_frameworks": self.requirements.fallback_frameworks
            },
            "context": self.context,
            "metadata": {
                "created_at": self.metadata.created_at.isoformat(),
                "started_at": self.metadata.started_at.isoformat() if self.metadata.started_at else None,
                "completed_at": self.metadata.completed_at.isoformat() if self.metadata.completed_at else None,
                "execution_time": self.metadata.execution_time,
                "retries": self.metadata.retries,
                "max_retries": self.metadata.max_retries
            },
            "error": self.error,
            "result": self.result.to_dict() if self.result else None
        }
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'UniversalTask':
        """从字典创建任务"""
        task = cls(
            content=data["content"],
            task_type=TaskType(data["task_type"]),
            priority=TaskPriority(data["priority"]),
            task_id=data.get("id")
        )
        
        # 设置需求
        req_data = data.get("requirements", {})
        task.requirements = TaskRequirements(
            capabilities=req_data.get("capabilities", []),
            max_execution_time=req_data.get("max_execution_time"),
            memory_limit=req_data.get("memory_limit"),
            preferred_framework=req_data.get("preferred_framework"),
            fallback_frameworks=req_data.get("fallback_frameworks", [])
        )
        
        # 设置上下文
        task.context = data.get("context", {})
        
        # 设置状态
        task.status = TaskStatus(data["status"])
        
        # 设置元数据
        meta_data = data.get("metadata", {})
        task.metadata = TaskMetadata(
            created_at=datetime.fromisoformat(meta_data["created_at"]) if meta_data.get("created_at") else datetime.now(),
            started_at=datetime.fromisoformat(meta_data["started_at"]) if meta_data.get("started_at") else None,
            completed_at=datetime.fromisoformat(meta_data["completed_at"]) if meta_data.get("completed_at") else None,
            execution_time=meta_data.get("execution_time"),
            retries=meta_data.get("retries", 0),
            max_retries=meta_data.get("max_retries", 3)
        )
        
        # 设置错误和结果
        task.error = data.get("error")
        if data.get("result"):
            from .result import UniversalResult
            task.result = UniversalResult.from_dict(data["result"])
        
        return task
    
    def __str__(self) -> str:
        return f"UniversalTask(id='{self.id}', type='{self.task_type.value}', status='{self.status.value}', content='{self.content[:50]}...')"
    
    def __repr__(self) -> str:
        return self.__str__()
    
    # 增强的任务执行流程方法
    
    def validate(self) -> List[str]:
        """验证任务是否有效"""
        errors = []
        
        if not self.content.strip():
            errors.append("Task content cannot be empty")
        
        if self.requirements.max_execution_time is not None and self.requirements.max_execution_time <= 0:
            errors.append("Max execution time must be positive")
        
        if self.requirements.memory_limit is not None and self.requirements.memory_limit <= 0:
            errors.append("Memory limit must be positive")
        
        return errors
    
    def is_valid(self) -> bool:
        """检查任务是否有效"""
        return len(self.validate()) == 0
    
    def can_execute(self) -> bool:
        """检查任务是否可以执行"""
        return (
            self.status in [TaskStatus.PENDING, TaskStatus.RUNNING] and
            self.is_valid() and
            self.metadata.retries <= self.metadata.max_retries
        )
    
    def is_timeout(self) -> bool:
        """检查任务是否超时"""
        if not self.requirements.max_execution_time or not self.metadata.started_at:
            return False
        
        elapsed = (datetime.now() - self.metadata.started_at).total_seconds()
        return elapsed > self.requirements.max_execution_time
    
    def get_elapsed_time(self) -> Optional[float]:
        """获取已执行时间（秒）"""
        if not self.metadata.started_at:
            return None
        
        end_time = self.metadata.completed_at or datetime.now()
        return (end_time - self.metadata.started_at).total_seconds()
    
    def get_remaining_time(self) -> Optional[float]:
        """获取剩余执行时间（秒）"""
        if not self.requirements.max_execution_time:
            return None
        
        elapsed = self.get_elapsed_time()
        if elapsed is None:
            return self.requirements.max_execution_time
        
        return max(0, self.requirements.max_execution_time - elapsed)
    
    def add_checkpoint(self, name: str, data: Dict[str, Any] = None) -> None:
        """添加检查点"""
        if not hasattr(self, 'checkpoints'):
            self.checkpoints = []
        
        checkpoint = {
            "name": name,
            "timestamp": datetime.now(),
            "data": data or {}
        }
        self.checkpoints.append(checkpoint)
    
    def get_checkpoints(self) -> List[Dict[str, Any]]:
        """获取所有检查点"""
        return getattr(self, 'checkpoints', [])
    
    def merge_context(self, other_context: Dict[str, Any]) -> None:
        """合并上下文信息"""
        if other_context:
            self.context.update(other_context) 


"""
业务能力层性能和压力测试
测试大规模并发、内存使用、响应时间、吞吐量等性能指标
"""

import pytest
import asyncio
import time
import psutil
import os
from datetime import datetime, timedelta
from unittest.mock import Mock, AsyncMock
from typing import List, Dict, Any
from concurrent.futures import ThreadPoolExecutor
import statistics
import gc

from layers.business.teams.collaboration_manager import (
    CollaborationManager, CollaborationPattern, CollaborationRole, TeamMember
)
from layers.business.teams.team_engine import TeamManager
from layers.business.workflows.workflow_engine import (
    WorkflowEngine, WorkflowDefinition, WorkflowStep, WorkflowStepType
)
from layers.business.communication_manager import BusinessCommunicationManager

from layers.framework.abstractions.agent import UniversalAgent, AgentCapability
from layers.framework.abstractions.task import UniversalTask, TaskType, TaskPriority
from layers.framework.abstractions.context import UniversalContext
from layers.framework.abstractions.result import UniversalResult, ResultStatus


class PerformanceMetrics:
    """性能指标收集器"""
    
    def __init__(self):
        self.start_time = None
        self.end_time = None
        self.response_times = []
        self.memory_snapshots = []
        self.cpu_snapshots = []
        self.success_count = 0
        self.failure_count = 0
        self.process = psutil.Process(os.getpid())
    
    def start_monitoring(self):
        """开始性能监控"""
        self.start_time = time.time()
        self.memory_snapshots = [self.process.memory_info().rss]
        self.cpu_snapshots = [self.process.cpu_percent()]
    
    def record_operation(self, start_time: float, end_time: float, success: bool):
        """记录操作性能"""
        response_time = end_time - start_time
        self.response_times.append(response_time)
        
        if success:
            self.success_count += 1
        else:
            self.failure_count += 1
        
        # 记录内存和CPU快照
        self.memory_snapshots.append(self.process.memory_info().rss)
        self.cpu_snapshots.append(self.process.cpu_percent())
    
    def stop_monitoring(self):
        """停止性能监控"""
        self.end_time = time.time()
    
    def get_metrics(self) -> Dict[str, Any]:
        """获取性能指标"""
        total_time = self.end_time - self.start_time if self.end_time else 0
        total_operations = self.success_count + self.failure_count
        
        return {
            "total_time": total_time,
            "total_operations": total_operations,
            "success_rate": self.success_count / total_operations if total_operations > 0 else 0,
            "throughput": total_operations / total_time if total_time > 0 else 0,
            "response_times": {
                "min": min(self.response_times) if self.response_times else 0,
                "max": max(self.response_times) if self.response_times else 0,
                "avg": statistics.mean(self.response_times) if self.response_times else 0,
                "median": statistics.median(self.response_times) if self.response_times else 0,
                "p95": statistics.quantiles(self.response_times, n=20)[18] if len(self.response_times) > 20 else 0,
                "p99": statistics.quantiles(self.response_times, n=100)[98] if len(self.response_times) > 100 else 0
            },
            "memory": {
                "initial": self.memory_snapshots[0] if self.memory_snapshots else 0,
                "final": self.memory_snapshots[-1] if self.memory_snapshots else 0,
                "peak": max(self.memory_snapshots) if self.memory_snapshots else 0,
                "increase": (self.memory_snapshots[-1] - self.memory_snapshots[0]) if len(self.memory_snapshots) > 1 else 0
            },
            "cpu": {
                "avg": statistics.mean(self.cpu_snapshots) if self.cpu_snapshots else 0,
                "peak": max(self.cpu_snapshots) if self.cpu_snapshots else 0
            }
        }


class TestCollaborationManagerPerformance:
    """协作管理器性能测试"""
    
    @pytest.fixture
    def collaboration_manager(self):
        return CollaborationManager()
    
    @pytest.fixture
    def performance_agents(self):
        """创建性能测试用的Agent池"""
        agents = []
        for i in range(100):  # 100个Agent
            agent = Mock(spec=UniversalAgent)
            agent.agent_id = f"perf_agent_{i}"
            agent.name = f"Performance Agent {i}"
            agent.capabilities = [
                AgentCapability.TEXT_PROCESSING,
                AgentCapability.REASONING,
                AgentCapability.DATA_ANALYSIS
            ]
            
            async def mock_execute(task, processing_time=0.01 + (i % 10) * 0.001):
                await asyncio.sleep(processing_time)  # 模拟不同的处理时间
                return UniversalResult(
                    task_id=task.task_id,
                    status=ResultStatus.SUCCESS,
                    data={"processed": True, "agent_id": agent.agent_id},
                    metadata={"processing_time": processing_time}
                )
            
            agent.execute = mock_execute
            agents.append(agent)
        
        return agents

    @pytest.mark.asyncio
    async def test_large_team_collaboration_performance(self, collaboration_manager, performance_agents):
        """测试大团队协作性能"""
        metrics = PerformanceMetrics()
        metrics.start_monitoring()
        
        # 创建大团队（50个成员）
        team_id = "large_team_performance"
        large_team_agents = performance_agents[:50]
        
        members = [
            TeamMember(
                agent=agent,
                role=CollaborationRole.CONTRIBUTOR,
                capabilities=agent.capabilities,
                priority=i % 5
            )
            for i, agent in enumerate(large_team_agents)
        ]
        
        start_time = time.time()
        await collaboration_manager.create_team(team_id, members)
        end_time = time.time()
        
        metrics.record_operation(start_time, end_time, True)
        
        # 执行大规模并行协作
        task = UniversalTask(
            task_id="large_team_task",
            task_type=TaskType.ANALYSIS,
            description="Large team collaboration performance test",
            priority=TaskPriority.HIGH
        )
        
        start_time = time.time()
        result = await collaboration_manager.collaborate(
            team_id=team_id,
            task=task,
            pattern=CollaborationPattern.PARALLEL
        )
        end_time = time.time()
        
        metrics.record_operation(start_time, end_time, 
                                result.final_result.status == ResultStatus.SUCCESS)
        
        metrics.stop_monitoring()
        performance_data = metrics.get_metrics()
        
        # 性能断言
        assert performance_data["success_rate"] >= 0.95  # 95%成功率
        assert performance_data["response_times"]["avg"] < 5.0  # 平均响应时间小于5秒
        assert performance_data["memory"]["increase"] < 100 * 1024 * 1024  # 内存增长小于100MB
        
        # 验证大团队协作结果
        assert len(result.individual_results) == 50
        assert result.final_result.status == ResultStatus.SUCCESS

    @pytest.mark.asyncio
    async def test_concurrent_collaborations_performance(self, collaboration_manager, performance_agents):
        """测试并发协作性能"""
        metrics = PerformanceMetrics()
        metrics.start_monitoring()
        
        # 创建多个团队
        teams = []
        team_size = 5
        num_teams = 20
        
        for i in range(num_teams):
            team_id = f"concurrent_team_{i}"
            team_agents = performance_agents[i*team_size:(i+1)*team_size]
            
            members = [
                TeamMember(
                    agent=agent,
                    role=CollaborationRole.CONTRIBUTOR,
                    capabilities=agent.capabilities
                )
                for agent in team_agents
            ]
            
            await collaboration_manager.create_team(team_id, members)
            teams.append(team_id)
        
        # 并发执行协作任务
        collaboration_tasks = []
        
        for i, team_id in enumerate(teams):
            task = UniversalTask(
                task_id=f"concurrent_task_{i}",
                task_type=TaskType.ANALYSIS,
                description=f"Concurrent collaboration task {i}",
                priority=TaskPriority.MEDIUM
            )
            
            # 随机选择协作模式
            patterns = [
                CollaborationPattern.PARALLEL,
                CollaborationPattern.SEQUENTIAL,
                CollaborationPattern.EXPERT_CONSULTATION,
                CollaborationPattern.ROUND_ROBIN
            ]
            pattern = patterns[i % len(patterns)]
            
            coroutine = collaboration_manager.collaborate(
                team_id=team_id,
                task=task,
                pattern=pattern
            )
            collaboration_tasks.append((coroutine, time.time()))
        
        # 执行所有并发任务
        start_time = time.time()
        results = await asyncio.gather(*[task[0] for task in collaboration_tasks])
        end_time = time.time()
        
        # 记录每个任务的性能
        for i, result in enumerate(results):
            task_start_time = collaboration_tasks[i][1]
            success = result.final_result.status == ResultStatus.SUCCESS
            metrics.record_operation(task_start_time, end_time, success)
        
        metrics.stop_monitoring()
        performance_data = metrics.get_metrics()
        
        # 性能断言
        assert performance_data["success_rate"] >= 0.90  # 90%成功率
        assert performance_data["response_times"]["p95"] < 10.0  # 95%的请求在10秒内完成
        assert performance_data["throughput"] >= 2.0  # 至少每秒处理2个协作任务
        
        # 验证并发协作结果
        success_count = sum(1 for result in results 
                           if result.final_result.status == ResultStatus.SUCCESS)
        assert success_count >= num_teams * 0.9

    @pytest.mark.asyncio
    async def test_collaboration_pattern_performance_comparison(self, collaboration_manager, performance_agents):
        """测试不同协作模式的性能对比"""
        team_id = "pattern_performance_team"
        team_agents = performance_agents[:10]
        
        members = [
            TeamMember(
                agent=agent,
                role=CollaborationRole.CONTRIBUTOR,
                capabilities=agent.capabilities
            )
            for agent in team_agents
        ]
        
        await collaboration_manager.create_team(team_id, members)
        
        # 测试不同协作模式的性能
        patterns_performance = {}
        
        patterns_to_test = [
            CollaborationPattern.SEQUENTIAL,
            CollaborationPattern.PARALLEL,
            CollaborationPattern.ROUND_ROBIN,
            CollaborationPattern.EXPERT_CONSULTATION,
            CollaborationPattern.CONSENSUS
        ]
        
        for pattern in patterns_to_test:
            pattern_metrics = PerformanceMetrics()
            pattern_metrics.start_monitoring()
            
            # 为每种模式执行多次测试
            for i in range(5):
                task = UniversalTask(
                    task_id=f"pattern_test_{pattern.value}_{i}",
                    task_type=TaskType.ANALYSIS,
                    description=f"Pattern performance test for {pattern.value}"
                )
                
                start_time = time.time()
                result = await collaboration_manager.collaborate(
                    team_id=team_id,
                    task=task,
                    pattern=pattern
                )
                end_time = time.time()
                
                success = result.final_result.status == ResultStatus.SUCCESS
                pattern_metrics.record_operation(start_time, end_time, success)
            
            pattern_metrics.stop_monitoring()
            patterns_performance[pattern.value] = pattern_metrics.get_metrics()
        
        # 分析性能差异
        for pattern_name, perf_data in patterns_performance.items():
            print(f"\n{pattern_name} Performance:")
            print(f"  Average Response Time: {perf_data['response_times']['avg']:.3f}s")
            print(f"  Success Rate: {perf_data['success_rate']:.2%}")
            print(f"  Throughput: {perf_data['throughput']:.2f} ops/sec")
        
        # 验证性能合理性
        for pattern_name, perf_data in patterns_performance.items():
            assert perf_data["success_rate"] >= 0.8  # 所有模式至少80%成功率
            
            # 并行模式应该最快
            if pattern_name == "parallel":
                assert perf_data["response_times"]["avg"] < 2.0
            
            # 顺序模式可能较慢但应该稳定
            if pattern_name == "sequential":
                assert perf_data["success_rate"] >= 0.95


class TestWorkflowEnginePerformance:
    """工作流引擎性能测试"""
    
    @pytest.fixture
    def workflow_engine(self):
        return WorkflowEngine()

    @pytest.mark.asyncio
    async def test_complex_workflow_performance(self, workflow_engine, performance_agents):
        """测试复杂工作流性能"""
        metrics = PerformanceMetrics()
        metrics.start_monitoring()
        
        # 注册Agent
        for agent in performance_agents[:20]:
            await workflow_engine.register_agent(agent)
        
        # 创建复杂工作流（多层依赖）
        complex_steps = []
        
        # 第一层：初始化步骤
        for i in range(3):
            step = WorkflowStep(
                step_id=f"init_{i}",
                name=f"Initialization {i}",
                step_type=WorkflowStepType.AGENT_TASK,
                config={"agent_id": f"perf_agent_{i}"},
                timeout=30
            )
            complex_steps.append(step)
        
        # 第二层：依赖第一层的步骤
        for i in range(5):
            step = WorkflowStep(
                step_id=f"process_{i}",
                name=f"Processing {i}",
                step_type=WorkflowStepType.AGENT_TASK,
                config={"agent_id": f"perf_agent_{i+3}"},
                dependencies=[f"init_{i%3}"],
                timeout=30
            )
            complex_steps.append(step)
        
        # 第三层：并行处理步骤
        parallel_step = WorkflowStep(
            step_id="parallel_processing",
            name="Parallel Processing",
            step_type=WorkflowStepType.PARALLEL,
            config={
                "parallel_steps": [
                    {
                        "step_id": f"parallel_{i}",
                        "step_type": "agent_task",
                        "config": {"agent_id": f"perf_agent_{i+8}"}
                    }
                    for i in range(6)
                ]
            },
            dependencies=[f"process_{i}" for i in range(5)],
            timeout=60
        )
        complex_steps.append(parallel_step)
        
        # 第四层：最终汇总
        final_step = WorkflowStep(
            step_id="finalization",
            name="Final Processing",
            step_type=WorkflowStepType.AGENT_TASK,
            config={"agent_id": "perf_agent_19"},
            dependencies=["parallel_processing"],
            timeout=30
        )
        complex_steps.append(final_step)
        
        complex_workflow = WorkflowDefinition(
            workflow_id="complex_performance_workflow",
            name="Complex Performance Workflow",
            steps=complex_steps,
            timeout=300
        )
        
        # 注册工作流
        start_time = time.time()
        await workflow_engine.register_workflow(complex_workflow)
        end_time = time.time()
        metrics.record_operation(start_time, end_time, True)
        
        # 执行复杂工作流
        task = UniversalTask(
            task_id="complex_performance_task",
            task_type=TaskType.WORKFLOW,
            description="Complex workflow performance test"
        )
        
        start_time = time.time()
        execution_id = await workflow_engine.start_workflow("complex_performance_workflow", task)
        execution = await workflow_engine.wait_for_completion(execution_id, timeout=60)
        end_time = time.time()
        
        success = execution.status.value == "completed"
        metrics.record_operation(start_time, end_time, success)
        
        metrics.stop_monitoring()
        performance_data = metrics.get_metrics()
        
        # 性能断言
        assert performance_data["success_rate"] >= 0.95
        assert performance_data["response_times"]["avg"] < 30.0  # 平均30秒内完成
        assert len(execution.step_executions) >= 10  # 至少执行了10个步骤

    @pytest.mark.asyncio
    async def test_concurrent_workflow_executions(self, workflow_engine, performance_agents):
        """测试并发工作流执行性能"""
        metrics = PerformanceMetrics()
        metrics.start_monitoring()
        
        # 注册Agent
        for agent in performance_agents[:50]:
            await workflow_engine.register_agent(agent)
        
        # 创建简单但资源密集的工作流
        intensive_workflow = WorkflowDefinition(
            workflow_id="intensive_workflow",
            name="Resource Intensive Workflow",
            steps=[
                WorkflowStep(
                    step_id="intensive_processing",
                    name="Intensive Processing",
                    step_type=WorkflowStepType.PARALLEL,
                    config={
                        "parallel_steps": [
                            {
                                "step_id": f"intensive_{i}",
                                "step_type": "agent_task",
                                "config": {"agent_id": f"perf_agent_{i}"}
                            }
                            for i in range(10)
                        ]
                    },
                    timeout=60
                )
            ],
            timeout=120
        )
        
        await workflow_engine.register_workflow(intensive_workflow)
        
        # 并发启动多个工作流
        num_concurrent = 20
        execution_tasks = []
        
        for i in range(num_concurrent):
            task = UniversalTask(
                task_id=f"concurrent_workflow_task_{i}",
                task_type=TaskType.WORKFLOW,
                description=f"Concurrent workflow execution {i}"
            )
            
            start_time = time.time()
            coroutine = workflow_engine.start_workflow("intensive_workflow", task)
            execution_tasks.append((coroutine, start_time))
        
        # 启动所有工作流
        execution_ids = []
        for coroutine, start_time in execution_tasks:
            execution_id = await coroutine
            execution_ids.append((execution_id, start_time))
        
        # 等待所有完成
        completion_tasks = []
        for execution_id, start_time in execution_ids:
            coroutine = workflow_engine.wait_for_completion(execution_id, timeout=30)
            completion_tasks.append((coroutine, start_time))
        
        # 收集结果
        for coroutine, start_time in completion_tasks:
            execution = await coroutine
            end_time = time.time()
            success = execution.status.value == "completed"
            metrics.record_operation(start_time, end_time, success)
        
        metrics.stop_monitoring()
        performance_data = metrics.get_metrics()
        
        # 性能断言
        assert performance_data["success_rate"] >= 0.85  # 85%成功率
        assert performance_data["throughput"] >= 1.0  # 至少每秒1个工作流
        assert performance_data["response_times"]["p95"] < 25.0  # 95%在25秒内完成

    @pytest.mark.asyncio
    async def test_workflow_memory_scalability(self, workflow_engine, performance_agents):
        """测试工作流内存可扩展性"""
        initial_memory = psutil.Process().memory_info().rss
        
        # 注册Agent
        for agent in performance_agents[:30]:
            await workflow_engine.register_agent(agent)
        
        # 创建内存密集型工作流
        memory_steps = []
        for i in range(100):  # 100步工作流
            step = WorkflowStep(
                step_id=f"memory_step_{i}",
                name=f"Memory Step {i}",
                step_type=WorkflowStepType.AGENT_TASK,
                config={
                    "agent_id": f"perf_agent_{i % 30}",
                    "large_data": "x" * 1000  # 模拟大数据
                },
                dependencies=[f"memory_step_{i-1}"] if i > 0 else [],
                timeout=5
            )
            memory_steps.append(step)
        
        memory_workflow = WorkflowDefinition(
            workflow_id="memory_scalability_workflow",
            name="Memory Scalability Workflow",
            steps=memory_steps,
            timeout=600
        )
        
        await workflow_engine.register_workflow(memory_workflow)
        
        # 执行内存密集型工作流
        task = UniversalTask(
            task_id="memory_scalability_task",
            task_type=TaskType.WORKFLOW,
            description="Memory scalability test"
        )
        
        execution_id = await workflow_engine.start_workflow("memory_scalability_workflow", task)
        execution = await workflow_engine.wait_for_completion(execution_id, timeout=120)
        
        final_memory = psutil.Process().memory_info().rss
        memory_increase = final_memory - initial_memory
        
        # 内存使用断言
        assert memory_increase < 500 * 1024 * 1024  # 内存增长小于500MB
        assert execution.status.value == "completed"
        
        # 强制垃圾回收
        gc.collect()
        
        # 检查内存是否有效释放
        post_gc_memory = psutil.Process().memory_info().rss
        memory_after_gc = post_gc_memory - initial_memory
        assert memory_after_gc < memory_increase * 0.8  # GC后内存应该减少


class TestTeamManagerPerformance:
    """团队管理器性能测试"""
    
    @pytest.fixture
    def team_manager(self):
        return TeamManager()

    @pytest.mark.asyncio
    async def test_large_scale_team_management(self, team_manager, performance_agents):
        """测试大规模团队管理性能"""
        metrics = PerformanceMetrics()
        metrics.start_monitoring()
        
        # 创建100个团队，每个团队5-15个成员
        num_teams = 100
        teams_created = []
        
        for i in range(num_teams):
            team_id = f"large_scale_team_{i}"
            team_size = 5 + (i % 11)  # 5-15个成员
            team_agents = performance_agents[i*2:(i*2)+team_size]
            
            start_time = time.time()
            await team_manager.create_team(
                team_id=team_id,
                team_name=f"Large Scale Team {i}",
                agents=team_agents,
                team_config={"max_size": 20}
            )
            end_time = time.time()
            
            metrics.record_operation(start_time, end_time, True)
            teams_created.append(team_id)
        
        # 并发执行团队任务
        team_tasks = []
        for i, team_id in enumerate(teams_created[:50]):  # 选择50个团队
            task = UniversalTask(
                task_id=f"large_scale_task_{i}",
                task_type=TaskType.ANALYSIS,
                description=f"Large scale team task {i}"
            )
            
            coroutine = team_manager.execute_team_task(
                team_id, task, CollaborationPattern.PARALLEL
            )
            team_tasks.append((coroutine, time.time()))
        
        # 执行所有团队任务
        results = []
        for coroutine, start_time in team_tasks:
            result = await coroutine
            end_time = time.time()
            success = result.final_result.status == ResultStatus.SUCCESS
            metrics.record_operation(start_time, end_time, success)
            results.append(result)
        
        metrics.stop_monitoring()
        performance_data = metrics.get_metrics()
        
        # 性能断言
        assert performance_data["success_rate"] >= 0.90
        assert performance_data["throughput"] >= 10.0  # 每秒至少10个操作
        assert len(teams_created) == num_teams
        
        success_results = sum(1 for result in results 
                             if result.final_result.status == ResultStatus.SUCCESS)
        assert success_results >= len(results) * 0.9

    @pytest.mark.asyncio
    async def test_team_optimization_performance(self, team_manager, performance_agents):
        """测试团队优化性能"""
        metrics = PerformanceMetrics()
        metrics.start_monitoring()
        
        # 创建需要优化的团队
        team_id = "optimization_performance_team"
        
        # 创建性能参差不齐的Agent
        mixed_agents = []
        for i, agent in enumerate(performance_agents[:20]):
            # 设置不同的性能分数
            agent.performance_score = 0.3 + (i % 5) * 0.15
            mixed_agents.append(agent)
        
        await team_manager.create_team(
            team_id=team_id,
            team_name="Optimization Performance Team",
            agents=mixed_agents
        )
        
        # 执行基准任务
        baseline_tasks = []
        for i in range(10):
            task = UniversalTask(
                task_id=f"baseline_task_{i}",
                task_type=TaskType.ANALYSIS,
                description=f"Baseline task {i}"
            )
            
            start_time = time.time()
            result = await team_manager.execute_team_task(
                team_id, task, CollaborationPattern.ROUND_ROBIN
            )
            end_time = time.time()
            
            success = result.final_result.status == ResultStatus.SUCCESS
            metrics.record_operation(start_time, end_time, success)
        
        # 执行团队优化
        start_time = time.time()
        optimization_suggestions = await team_manager.get_optimization_suggestions(team_id)
        end_time = time.time()
        metrics.record_operation(start_time, end_time, True)
        
        # 应用优化建议
        if optimization_suggestions.get("suggested_additions"):
            high_performance_agents = [agent for agent in performance_agents[20:25] 
                                     if not hasattr(agent, 'performance_score')]
            for agent in high_performance_agents:
                agent.performance_score = 0.9
            
            for agent in high_performance_agents[:2]:
                start_time = time.time()
                await team_manager.add_team_member(
                    team_id, agent, CollaborationRole.EXPERT
                )
                end_time = time.time()
                metrics.record_operation(start_time, end_time, True)
        
        # 执行优化后任务
        for i in range(10):
            task = UniversalTask(
                task_id=f"optimized_task_{i}",
                task_type=TaskType.ANALYSIS,
                description=f"Optimized task {i}"
            )
            
            start_time = time.time()
            result = await team_manager.execute_team_task(
                team_id, task, CollaborationPattern.EXPERT_CONSULTATION
            )
            end_time = time.time()
            
            success = result.final_result.status == ResultStatus.SUCCESS
            metrics.record_operation(start_time, end_time, success)
        
        metrics.stop_monitoring()
        performance_data = metrics.get_metrics()
        
        # 性能断言
        assert performance_data["success_rate"] >= 0.85
        assert performance_data["response_times"]["avg"] < 5.0


class TestStressTests:
    """压力测试"""

    @pytest.mark.asyncio
    async def test_system_overload_handling(self, performance_agents):
        """测试系统过载处理"""
        # 创建所有组件
        collaboration_manager = CollaborationManager()
        team_manager = TeamManager()
        workflow_engine = WorkflowEngine()
        
        metrics = PerformanceMetrics()
        metrics.start_monitoring()
        
        # 注册大量Agent
        for agent in performance_agents:
            await workflow_engine.register_agent(agent)
        
        # 创建过载场景：大量团队、工作流、协作任务
        overload_tasks = []
        
        # 1. 创建大量团队
        for i in range(50):
            team_id = f"overload_team_{i}"
            team_agents = performance_agents[i*2:(i*2)+5]
            
            members = [
                TeamMember(agent=agent, role=CollaborationRole.CONTRIBUTOR,
                          capabilities=agent.capabilities)
                for agent in team_agents
            ]
            
            task = collaboration_manager.create_team(team_id, members)
            overload_tasks.append(task)
        
        # 等待团队创建完成
        await asyncio.gather(*overload_tasks)
        
        # 2. 创建大量工作流
        for i in range(20):
            workflow = WorkflowDefinition(
                workflow_id=f"overload_workflow_{i}",
                name=f"Overload Workflow {i}",
                steps=[
                    WorkflowStep(
                        step_id="overload_step",
                        name="Overload Step",
                        step_type=WorkflowStepType.PARALLEL,
                        config={
                            "parallel_steps": [
                                {"step_id": f"sub_{j}", "step_type": "agent_task",
                                 "config": {"agent_id": f"perf_agent_{j}"}}
                                for j in range(10)
                            ]
                        },
                        timeout=30
                    )
                ]
            )
            await workflow_engine.register_workflow(workflow)
        
        # 3. 同时执行大量任务
        concurrent_tasks = []
        
        # 协作任务
        for i in range(100):
            team_id = f"overload_team_{i % 50}"
            task = UniversalTask(
                task_id=f"overload_collab_task_{i}",
                task_type=TaskType.ANALYSIS,
                description=f"Overload collaboration task {i}"
            )
            
            coroutine = collaboration_manager.collaborate(
                team_id, task, CollaborationPattern.PARALLEL
            )
            concurrent_tasks.append(coroutine)
        
        # 工作流任务
        for i in range(50):
            workflow_id = f"overload_workflow_{i % 20}"
            task = UniversalTask(
                task_id=f"overload_workflow_task_{i}",
                task_type=TaskType.WORKFLOW,
                description=f"Overload workflow task {i}"
            )
            
            async def execute_workflow(wf_id, wf_task):
                execution_id = await workflow_engine.start_workflow(wf_id, wf_task)
                return await workflow_engine.wait_for_completion(execution_id, timeout=60)
            
            coroutine = execute_workflow(workflow_id, task)
            concurrent_tasks.append(coroutine)
        
        # 执行所有任务并记录性能
        start_time = time.time()
        
        try:
            # 使用超时来避免无限等待
            results = await asyncio.wait_for(
                asyncio.gather(*concurrent_tasks, return_exceptions=True),
                timeout=120.0
            )
            
            end_time = time.time()
            
            # 统计成功和失败
            success_count = 0
            failure_count = 0
            
            for result in results:
                if isinstance(result, Exception):
                    failure_count += 1
                else:
                    if hasattr(result, 'final_result'):
                        # 协作结果
                        if result.final_result.status == ResultStatus.SUCCESS:
                            success_count += 1
                        else:
                            failure_count += 1
                    elif hasattr(result, 'status'):
                        # 工作流结果
                        if result.status.value == "completed":
                            success_count += 1
                        else:
                            failure_count += 1
                    else:
                        failure_count += 1
            
            metrics.record_operation(start_time, end_time, success_count > failure_count)
            
        except asyncio.TimeoutError:
            end_time = time.time()
            metrics.record_operation(start_time, end_time, False)
            success_count = 0
            failure_count = len(concurrent_tasks)
        
        metrics.stop_monitoring()
        performance_data = metrics.get_metrics()
        
        # 压力测试断言（更宽松的标准）
        total_tasks = len(concurrent_tasks)
        success_rate = success_count / total_tasks if total_tasks > 0 else 0
        
        # 在极端压力下，系统应该至少处理一部分任务
        assert success_rate >= 0.3  # 至少30%的任务成功
        assert performance_data["memory"]["increase"] < 1024 * 1024 * 1024  # 内存增长小于1GB
        
        print(f"Stress Test Results:")
        print(f"  Total Tasks: {total_tasks}")
        print(f"  Success Rate: {success_rate:.2%}")
        print(f"  Memory Increase: {performance_data['memory']['increase'] / 1024 / 1024:.1f}MB")
        print(f"  Peak CPU: {performance_data['cpu']['peak']:.1f}%")

    @pytest.mark.asyncio
    async def test_resource_exhaustion_recovery(self, performance_agents):
        """测试资源耗尽后的恢复能力"""
        collaboration_manager = CollaborationManager()
        
        # 创建资源耗尽场景
        team_id = "resource_exhaustion_team"
        members = [
            TeamMember(agent=agent, role=CollaborationRole.CONTRIBUTOR,
                      capabilities=agent.capabilities)
            for agent in performance_agents[:50]  # 大团队
        ]
        
        await collaboration_manager.create_team(team_id, members)
        
        # 第一阶段：资源耗尽
        exhaustion_tasks = []
        for i in range(200):  # 大量任务
            task = UniversalTask(
                task_id=f"exhaustion_task_{i}",
                task_type=TaskType.ANALYSIS,
                description=f"Resource exhaustion task {i}"
            )
            
            coroutine = collaboration_manager.collaborate(
                team_id, task, CollaborationPattern.PARALLEL
            )
            exhaustion_tasks.append(coroutine)
        
        # 尝试执行所有任务（预期会有部分失败）
        try:
            await asyncio.wait_for(
                asyncio.gather(*exhaustion_tasks, return_exceptions=True),
                timeout=60.0
            )
        except asyncio.TimeoutError:
            pass  # 预期的超时
        
        # 第二阶段：恢复测试
        await asyncio.sleep(2.0)  # 等待系统恢复
        
        # 执行少量任务验证恢复
        recovery_tasks = []
        for i in range(5):
            task = UniversalTask(
                task_id=f"recovery_task_{i}",
                task_type=TaskType.ANALYSIS,
                description=f"Recovery test task {i}"
            )
            
            coroutine = collaboration_manager.collaborate(
                team_id, task, CollaborationPattern.SEQUENTIAL
            )
            recovery_tasks.append(coroutine)
        
        # 恢复阶段应该成功
        recovery_results = await asyncio.gather(*recovery_tasks, return_exceptions=True)
        
        recovery_success_count = 0
        for result in recovery_results:
            if not isinstance(result, Exception) and \
               hasattr(result, 'final_result') and \
               result.final_result.status == ResultStatus.SUCCESS:
                recovery_success_count += 1
        
        # 恢复断言
        recovery_rate = recovery_success_count / len(recovery_tasks)
        assert recovery_rate >= 0.8  # 恢复后至少80%成功率
        
        print(f"Recovery Test Results:")
        print(f"  Recovery Success Rate: {recovery_rate:.2%}")


if __name__ == "__main__":
    pytest.main([__file__, "-v", "--tb=short", "-s"]) 


#!/usr/bin/env python3
"""
智能上下文层基础测试

测试智能上下文层的四大核心组件功能
"""

import asyncio
import sys
import os

# 添加项目根目录到路径
sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))

from layers.intelligent_context import IntelligentContextLayer
from layers.framework.abstractions.context import UniversalContext
from layers.framework.abstractions.task import UniversalTask, TaskPriority


async def test_intelligent_context_layer():
    """测试智能上下文层基础功能"""
    
    print("🧩 开始测试智能上下文层...")
    
    # 1. 初始化智能上下文层
    print("\n1. 初始化智能上下文层")
    config = {
        'processing_mode': 'sequential',
        'context_engineering': {'enable_failure_detection': True},
        'rag_system': {'retrieval_strategy': 'semantic'},
        'knowledge_management': {'storage_type': 'vector_db'},
        'quality_control': {'quality_threshold': 0.7}
    }
    
    intelligent_context_layer = IntelligentContextLayer(config)
    print("✅ 智能上下文层初始化完成")
    
    # 2. 创建测试上下文
    print("\n2. 创建测试上下文")
    test_context = UniversalContext({
        'instructions': 'Complete the AI agent development task',
        'user_prompt': 'Implement the intelligent context layer',
        'state_history': ['Started project', 'Designed architecture'],
        'long_term_memory': {'project_goals': 'Build ADC framework'},
        'retrieved_info': 'Context engineering best practices',
        'available_tools': ['code_generator', 'documentation_tool']
    })
    test_context.set('context_id', 'test_context_001')
    test_context.set('created_at', '2025-08-22T10:00:00Z')
    test_context.set('priority', 'high')
    print("✅ 测试上下文创建完成")
    
    # 3. 创建测试任务
    print("\n3. 创建测试任务")
    test_task = UniversalTask(
        task_id="task_001",
        requirements=['Implement context engineering', 'Add RAG system', 'Ensure quality control'],
        priority=TaskPriority.HIGH,
        data={'task_type': 'development', 'complexity': 'high'},
        metadata={'assigned_to': 'intelligent_context_layer'}
    )
    print("✅ 测试任务创建完成")
    
    # 4. 执行智能上下文处理
    print("\n4. 执行智能上下文处理...")
    try:
        result = await intelligent_context_layer.process_context(test_context, test_task)
        
        print(f"✅ 处理完成！")
        print(f"   - 原始上下文ID: {result.original_context.get('context_id', 'unknown')}")
        print(f"   - 最终上下文ID: {result.quality_controlled_context.get('context_id', 'unknown')}")
        print(f"   - 处理时间: {result.processing_time:.3f}秒")
        print(f"   - 指标数量: {len(result.metrics)}")
        
        # 显示处理结果
        print(f"\n📊 处理指标:")
        for key, value in result.metrics.get('overall_metrics', {}).items():
            print(f"   - {key}: {value}")
            
    except Exception as e:
        print(f"❌ 处理失败: {e}")
        return False
    
    # 5. 测试组件状态
    print("\n5. 检查组件状态")
    status = intelligent_context_layer.get_status()
    print(f"✅ 层状态: {status['layer_name']}")
    print(f"   - 处理模式: {status['processing_mode']}")
    print(f"   - 已处理总数: {status['metrics']['total_processed']}")
    print(f"   - 平均处理时间: {status['metrics']['average_processing_time']:.3f}秒")
    print(f"   - 质量评分: {status['metrics']['quality_score']:.3f}")
    print(f"   - 效率评分: {status['metrics']['efficiency_score']:.3f}")
    
    # 6. 测试不同处理模式
    print("\n6. 测试并行处理模式")
    intelligent_context_layer.configure({'processing_mode': 'parallel'})
    
    try:
        parallel_result = await intelligent_context_layer.process_context(test_context, test_task)
        print(f"✅ 并行处理完成，处理时间: {parallel_result.processing_time:.3f}秒")
    except Exception as e:
        print(f"❌ 并行处理失败: {e}")
    
    # 7. 测试自适应处理模式
    print("\n7. 测试自适应处理模式")
    intelligent_context_layer.configure({'processing_mode': 'adaptive'})
    
    try:
        adaptive_result = await intelligent_context_layer.process_context(test_context, test_task)
        print(f"✅ 自适应处理完成，处理时间: {adaptive_result.processing_time:.3f}秒")
    except Exception as e:
        print(f"❌ 自适应处理失败: {e}")
    
    print("\n🎉 智能上下文层测试完成！")
    return True


async def test_individual_components():
    """测试各个组件的独立功能"""
    
    print("\n🔧 开始测试各个组件...")
    
    # 导入各个组件
    from layers.intelligent_context.context_engineering import ContextEngineering
    from layers.intelligent_context.rag_system import RAGSystem
    from layers.intelligent_context.knowledge_management import KnowledgeManagement
    from layers.intelligent_context.quality_control import QualityControl
    
    # 创建测试数据
    test_context = UniversalContext({
        'test': 'data', 
        'instructions': 'test instructions'
    })
    test_context.set('context_id', 'component_test_context')
    
    test_task = UniversalTask(
        task_id="component_test_task",
        requirements=['test requirement'],
        priority=TaskPriority.MEDIUM,
        data={},
        metadata={}
    )
    
    # 1. 测试上下文工程
    print("\n1. 测试上下文工程组件")
    try:
        context_engineering = ContextEngineering()
        engineered_context = await context_engineering.engineer_context(test_context, test_task)
        print(f"✅ 上下文工程完成: {engineered_context.get('context_id', 'unknown')}")
        print(f"   - 指标: {context_engineering.get_metrics()}")
    except Exception as e:
        print(f"❌ 上下文工程失败: {e}")
    
    # 2. 测试RAG系统
    print("\n2. 测试RAG系统组件")
    try:
        rag_system = RAGSystem()
        rag_enhanced_context = await rag_system.enhance_with_rag(test_context, test_task)
        print(f"✅ RAG增强完成: {rag_enhanced_context.get('context_id', 'unknown')}")
        print(f"   - 指标: {rag_system.get_metrics()}")
    except Exception as e:
        print(f"❌ RAG增强失败: {e}")
    
    # 3. 测试知识管理
    print("\n3. 测试知识管理组件")
    try:
        knowledge_management = KnowledgeManagement()
        managed_context = await knowledge_management.manage_knowledge(test_context, test_task)
        print(f"✅ 知识管理完成: {managed_context.get('context_id', 'unknown')}")
        print(f"   - 指标: {knowledge_management.get_metrics()}")
    except Exception as e:
        print(f"❌ 知识管理失败: {e}")
    
    # 4. 测试质量控制
    print("\n4. 测试质量控制组件")
    try:
        quality_control = QualityControl()
        controlled_context = await quality_control.control_quality(test_context, test_task)
        print(f"✅ 质量控制完成: {controlled_context.get('context_id', 'unknown')}")
        print(f"   - 指标: {quality_control.get_metrics()}")
    except Exception as e:
        print(f"❌ 质量控制失败: {e}")
    
    print("\n🎉 各组件测试完成！")


async def main():
    """主测试函数"""
    
    print("=" * 60)
    print("🚀 ADC智能上下文层测试")
    print("=" * 60)
    
    # 测试智能上下文层整体功能
    success = await test_intelligent_context_layer()
    
    if success:
        # 测试各个组件的独立功能
        await test_individual_components()
    
    print("\n" + "=" * 60)
    print("✅ 所有测试完成！")
    print("=" * 60)


if __name__ == "__main__":
    asyncio.run(main()) 


"""
A2A (Agent-to-Agent) Protocol Demo
演示Agent间通信协议的使用方法

这个示例展示了：
1. A2A协议的基本使用
2. Agent间握手和能力交换
3. 任务协作和结果分享
4. 多种传输方式的使用
"""

import asyncio
import sys
import os
from datetime import datetime, timedelta

# 添加项目根目录到Python路径
sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from layers.framework.abstractions.a2a_protocol import (
    A2AProtocolHandler,
    A2AAgentProfile,
    A2ACapability,
    A2ACapabilityType,
    A2AHTTPTransport,
    A2AWebSocketTransport,
    A2AMessageType,
    A2AMessage,
    A2AMessageHandler,
    create_a2a_capability,
    create_a2a_agent_profile
)


class TaskProcessingHandler(A2AMessageHandler):
    """任务处理消息处理器"""
    
    def __init__(self, agent_name: str):
        self.agent_name = agent_name
    
    async def handle_message(self, message: A2AMessage) -> A2AMessage:
        """处理任务请求消息"""
        if message.message_type == A2AMessageType.TASK_REQUEST:
            task_description = message.payload.get("task_description", "")
            task_data = message.payload.get("task_data", {})
            
            print(f"🔧 {self.agent_name} 收到任务请求: {task_description}")
            
            # 模拟任务处理
            await asyncio.sleep(0.5)
            
            # 创建任务响应
            response = A2AMessage(
                message_id=f"resp_{message.message_id}",
                protocol_version=message.protocol_version,
                message_type=A2AMessageType.TASK_RESPONSE,
                sender_id=message.receiver_id,
                receiver_id=message.sender_id,
                timestamp=datetime.now(),
                correlation_id=message.correlation_id,
                payload={
                    "task_result": f"任务 '{task_description}' 已完成",
                    "processed_data": {
                        "status": "completed",
                        "processing_time": "0.5s",
                        "result_quality": "high",
                        "output": f"由 {self.agent_name} 处理的结果"
                    },
                    "completion_time": datetime.now().isoformat()
                }
            )
            
            print(f"✅ {self.agent_name} 完成任务处理")
            return response
        
        return None


class CollaborationHandler(A2AMessageHandler):
    """协作消息处理器"""
    
    def __init__(self, agent_name: str):
        self.agent_name = agent_name
    
    async def handle_message(self, message: A2AMessage) -> A2AMessage:
        """处理协作邀请消息"""
        if message.message_type == A2AMessageType.COLLABORATION_INVITE:
            collaboration_type = message.payload.get("collaboration_type", "")
            collaboration_data = message.payload.get("collaboration_data", {})
            
            print(f"🤝 {self.agent_name} 收到协作邀请: {collaboration_type}")
            
            # 模拟协作决策
            await asyncio.sleep(0.2)
            
            # 接受协作
            response = A2AMessage(
                message_id=f"collab_resp_{message.message_id}",
                protocol_version=message.protocol_version,
                message_type=A2AMessageType.COLLABORATION_ACCEPT,
                sender_id=message.receiver_id,
                receiver_id=message.sender_id,
                timestamp=datetime.now(),
                correlation_id=message.correlation_id,
                payload={
                    "collaboration_id": message.payload.get("collaboration_id"),
                    "accepted": True,
                    "agent_role": "participant",
                    "available_time": "2h",
                    "commitment_level": "high"
                }
            )
            
            print(f"✅ {self.agent_name} 接受协作邀请")
            return response
        
        return None


async def create_demo_agent(agent_id: str, agent_name: str, transport_type: str = "http") -> A2AProtocolHandler:
    """创建演示用的Agent"""
    
    # 定义Agent能力
    capabilities = [
        create_a2a_capability(
            A2ACapabilityType.TEXT_PROCESSING,
            version="2.0",
            description="高级文本处理和分析",
            input_formats=["text", "markdown", "json"],
            output_formats=["text", "json", "html"]
        ),
        create_a2a_capability(
            A2ACapabilityType.CODE_GENERATION,
            version="1.5",
            description="Python和JavaScript代码生成",
            input_formats=["text", "json"],
            output_formats=["python", "javascript", "json"]
        ),
        create_a2a_capability(
            A2ACapabilityType.REASONING,
            version="1.0",
            description="逻辑推理和问题解决",
            input_formats=["text", "json"],
            output_formats=["text", "json"]
        )
    ]
    
    # 创建Agent配置文件
    profile = create_a2a_agent_profile(
        agent_id=agent_id,
        agent_name=agent_name,
        agent_type="multi_capability_agent",
        capabilities=capabilities,
        endpoint=f"http://localhost:8000/agents/{agent_id}"
    )
    
    # 选择传输方式
    if transport_type == "websocket":
        transport = A2AWebSocketTransport()
    else:
        transport = A2AHTTPTransport()
    
    # 创建协议处理器
    handler = A2AProtocolHandler(profile, transport)
    
    # 注册消息处理器
    handler.register_message_handler(A2AMessageType.TASK_REQUEST, TaskProcessingHandler(agent_name))
    handler.register_message_handler(A2AMessageType.COLLABORATION_INVITE, CollaborationHandler(agent_name))
    
    return handler


async def demo_basic_a2a_communication():
    """演示基本的A2A通信"""
    print("\n🚀 A2A协议基本通信演示")
    print("=" * 50)
    
    # 创建两个Agent
    agent1 = await create_demo_agent("agent_001", "Alice (文本专家)", "http")
    agent2 = await create_demo_agent("agent_002", "Bob (代码专家)", "websocket")
    
    print(f"📋 Agent 1: {agent1.agent_profile.agent_name}")
    print(f"   - ID: {agent1.agent_profile.agent_id}")
    print(f"   - 能力: {len(agent1.agent_profile.capabilities)} 个")
    print(f"   - 传输: HTTP")
    
    print(f"📋 Agent 2: {agent2.agent_profile.agent_name}")
    print(f"   - ID: {agent2.agent_profile.agent_id}")
    print(f"   - 能力: {len(agent2.agent_profile.capabilities)} 个")
    print(f"   - 传输: WebSocket")
    
    # 1. 握手连接
    print(f"\n🤝 步骤1: Agent间握手连接")
    success = await agent1.connect_to_agent("http://localhost:8001/agent_002")
    print(f"   连接结果: {'成功' if success else '失败'}")
    
    # 2. 能力交换
    print(f"\n🔄 步骤2: 能力交换")
    success = await agent1.send_capability_exchange(
        agent2.agent_profile.agent_id,
        agent2.agent_profile.endpoint
    )
    print(f"   能力交换: {'成功' if success else '失败'}")
    
    # 3. 任务请求
    print(f"\n📝 步骤3: 发送任务请求")
    correlation_id = await agent1.send_task_request(
        agent2.agent_profile.agent_id,
        agent2.agent_profile.endpoint,
        "生成一个Python函数来计算斐波那契数列",
        {
            "function_name": "fibonacci",
            "parameters": ["n"],
            "return_type": "int",
            "include_docstring": True
        }
    )
    print(f"   任务请求ID: {correlation_id}")
    
    # 4. 模拟任务响应处理
    print(f"\n⚡ 步骤4: 模拟任务响应")
    # 创建模拟的任务请求消息
    task_message = A2AMessage(
        message_id="task_001",
        protocol_version=agent1.agent_profile.supported_protocols[0],
        message_type=A2AMessageType.TASK_REQUEST,
        sender_id=agent1.agent_profile.agent_id,
        receiver_id=agent2.agent_profile.agent_id,
        timestamp=datetime.now(),
        correlation_id=correlation_id,
        payload={
            "task_description": "生成一个Python函数来计算斐波那契数列",
            "task_data": {
                "function_name": "fibonacci",
                "parameters": ["n"],
                "return_type": "int",
                "include_docstring": True
            }
        }
    )
    
    # Agent2处理任务
    response = await agent2.process_message(task_message)
    if response:
        print(f"   ✅ 任务响应: {response.payload.get('task_result', '无结果')}")
    
    # 5. 显示连接状态
    print(f"\n📊 步骤5: 连接状态")
    status1 = agent1.get_connection_status()
    status2 = agent2.get_connection_status()
    
    print(f"   Agent 1 状态: 已处理 {status1['messages_processed']} 条消息")
    print(f"   Agent 2 状态: 已处理 {status2['messages_processed']} 条消息")


async def demo_multi_agent_collaboration():
    """演示多Agent协作"""
    print("\n🤝 多Agent协作演示")
    print("=" * 50)
    
    # 创建三个专业Agent
    coordinator = await create_demo_agent("coordinator_001", "项目协调员 (Charlie)", "http")
    developer = await create_demo_agent("developer_001", "开发工程师 (David)", "websocket")
    tester = await create_demo_agent("tester_001", "测试工程师 (Eve)", "http")
    
    agents = [coordinator, developer, tester]
    
    print("📋 协作团队:")
    for agent in agents:
        print(f"   - {agent.agent_profile.agent_name} ({agent.agent_profile.agent_id})")
    
    # 发起协作邀请
    print(f"\n📨 发起协作邀请")
    collaboration_id = await coordinator.send_collaboration_invite(
        [developer.agent_profile.agent_id, tester.agent_profile.agent_id],
        "software_development_project",
        {
            "project_name": "A2A协议测试工具",
            "duration": "2h",
            "roles": {
                developer.agent_profile.agent_id: "lead_developer",
                tester.agent_profile.agent_id: "qa_engineer"
            },
            "deliverables": ["code_implementation", "test_cases", "documentation"]
        }
    )
    
    print(f"   协作ID: {collaboration_id}")
    
    # 模拟协作响应
    print(f"\n🔄 处理协作响应")
    
    # 创建协作邀请消息
    invite_message = A2AMessage(
        message_id="collab_invite_001",
        protocol_version=coordinator.agent_profile.supported_protocols[0],
        message_type=A2AMessageType.COLLABORATION_INVITE,
        sender_id=coordinator.agent_profile.agent_id,
        receiver_id=developer.agent_profile.agent_id,
        timestamp=datetime.now(),
        correlation_id=collaboration_id,
        payload={
            "collaboration_id": collaboration_id,
            "collaboration_type": "software_development_project",
            "collaboration_data": {
                "project_name": "A2A协议测试工具",
                "duration": "2h"
            },
            "participants": [developer.agent_profile.agent_id, tester.agent_profile.agent_id],
            "role_requirements": {"lead_developer": "Python开发经验"}
        }
    )
    
    # Developer响应
    dev_response = await developer.process_message(invite_message)
    if dev_response:
        print(f"   开发工程师响应: {dev_response.payload.get('accepted', False)}")
    
    # Tester响应（修改接收者ID）
    invite_message.receiver_id = tester.agent_profile.agent_id
    test_response = await tester.process_message(invite_message)
    if test_response:
        print(f"   测试工程师响应: {test_response.payload.get('accepted', False)}")
    
    # 显示协作状态
    print(f"\n📊 协作状态")
    coord_status = coordinator.get_connection_status()
    print(f"   协调员活跃协作: {coord_status['active_collaborations']} 个")
    print(f"   团队规模: {len(agents)} 个Agent")


async def demo_advanced_features():
    """演示高级功能"""
    print("\n⚡ A2A协议高级功能演示")
    print("=" * 50)
    
    # 创建专业Agent
    specialist = await create_demo_agent("specialist_001", "AI专家 (Frank)", "websocket")
    
    print("🔧 高级功能展示:")
    
    # 1. 心跳检测
    print(f"\n💓 心跳检测")
    heartbeat_msg = A2AMessage(
        message_id="heartbeat_001",
        protocol_version=specialist.agent_profile.supported_protocols[0],
        message_type=A2AMessageType.HEARTBEAT,
        sender_id="external_monitor",
        receiver_id=specialist.agent_profile.agent_id,
        timestamp=datetime.now(),
        payload={"ping": "alive_check"}
    )
    
    heartbeat_response = await specialist.process_message(heartbeat_msg)
    if heartbeat_response:
        print(f"   心跳响应: {heartbeat_response.payload.get('status', '无响应')}")
    
    # 2. 状态查询
    print(f"\n📊 状态查询")
    status_query_msg = A2AMessage(
        message_id="status_query_001",
        protocol_version=specialist.agent_profile.supported_protocols[0],
        message_type=A2AMessageType.STATUS_QUERY,
        sender_id="system_monitor",
        receiver_id=specialist.agent_profile.agent_id,
        timestamp=datetime.now(),
        payload={"query_type": "full_status"}
    )
    
    status_response = await specialist.process_message(status_query_msg)
    if status_response:
        status_data = status_response.payload
        print(f"   Agent状态: {status_data.get('agent_status', '未知')}")
        print(f"   当前负载: {status_data.get('current_load', 0)}")
        print(f"   可用能力: {status_data.get('available_capabilities', 0)} 个")
    
    # 3. 错误处理演示
    print(f"\n❌ 错误处理演示")
    invalid_msg = A2AMessage(
        message_id="invalid_001",
        protocol_version=specialist.agent_profile.supported_protocols[0],
        message_type=A2AMessageType.TASK_REQUEST,
        sender_id="error_generator",
        receiver_id=specialist.agent_profile.agent_id,
        timestamp=datetime.now(),
        payload={"invalid_data": None, "cause_error": True}
    )
    
    # 手动触发错误处理
    error_response = specialist._create_error_response(invalid_msg, "模拟处理错误")
    print(f"   错误响应: {error_response.payload.get('error_message', '无错误信息')}")


async def main():
    """主演示函数"""
    print("🌟 A2A (Agent-to-Agent) 协议演示")
    print("基于A2A协议标准实现的Agent间通信系统")
    print("=" * 60)
    
    try:
        # 基本通信演示
        await demo_basic_a2a_communication()
        
        # 等待一下，让输出更清晰
        await asyncio.sleep(1)
        
        # 多Agent协作演示
        await demo_multi_agent_collaboration()
        
        # 等待一下
        await asyncio.sleep(1)
        
        # 高级功能演示
        await demo_advanced_features()
        
        # 总结
        print("\n" + "=" * 60)
        print("🎉 A2A协议演示完成!")
        print("\n✨ 主要功能验证:")
        print("   ✅ Agent间握手和连接建立")
        print("   ✅ 能力交换和兼容性检查") 
        print("   ✅ 任务请求和响应处理")
        print("   ✅ 多Agent协作邀请和管理")
        print("   ✅ 心跳检测和状态监控")
        print("   ✅ 错误处理和异常管理")
        print("   ✅ HTTP和WebSocket传输支持")
        
        print("\n🚀 A2A协议已准备好用于生产环境的Agent间通信!")
        
    except Exception as e:
        print(f"\n💥 演示过程中发生错误: {str(e)}")
        import traceback
        traceback.print_exc()


if __name__ == "__main__":
    asyncio.run(main()) 


"""
Complete Agent Demo - 完整Agent示例
展示ADC系统的完整功能：从Agent创建到任务执行到结果输出
"""

import asyncio
import os
import sys
import json
from pathlib import Path
from datetime import datetime
from typing import Dict, Any, List, Optional

# 添加项目根目录到路径
project_root = Path(__file__).parent.parent
sys.path.insert(0, str(project_root))

from layers.framework.abstractions.agent import UniversalAgent, AgentCapability, AgentStatus
from layers.framework.abstractions.task import UniversalTask, TaskType, TaskStatus, TaskPriority
from layers.framework.abstractions.context import UniversalContext
from layers.framework.abstractions.result import UniversalResult, ResultStatus, ResultMetadata
from layers.adapter.deepseek.adapter import DeepSeekAdapter
from layers.business.workflows.workflow_engine import WorkflowEngine, WorkflowDefinition, WorkflowStep, WorkflowStepType
from layers.business.teams.collaboration_manager import CollaborationManager, CollaborationPattern
from layers.application.orchestration.orchestrator import ApplicationOrchestrator


class MockAgent(UniversalAgent):
    """模拟Agent，用于演示当真实Agent不可用时"""
    
    def __init__(self):
        super().__init__(
            name="模拟Agent",
            description="用于演示的模拟Agent",
            capabilities=[AgentCapability.CONVERSATION],
            config={}
        )
        self.agent_id = "mock_agent_001"  # 添加agent_id属性
    
    async def execute(self, task: UniversalTask, context: UniversalContext) -> UniversalResult:
        """模拟执行任务"""
        return UniversalResult(
            data="这是一个模拟的Agent响应结果，用于演示系统功能。",
            status=ResultStatus.SUCCESS,
            metadata=ResultMetadata(
                execution_time=0.1
            )
        )
    
    def get_schema(self) -> Dict[str, Any]:
        """获取Agent配置模式"""
        return {
            "type": "object",
            "properties": {
                "name": {"type": "string"},
                "type": {"type": "string", "enum": ["mock"]},
                "capabilities": {"type": "array", "items": {"type": "string"}}
            },
            "required": ["name", "type", "capabilities"]
        }
    
    def configure(self, config: Dict[str, Any]) -> None:
        """配置Agent"""
        if "name" in config:
            self.name = config["name"]
        if "description" in config:
            self.description = config["description"]
        if "capabilities" in config:
            # 将字符串转换为枚举
            caps = []
            for cap_str in config["capabilities"]:
                try:
                    cap = AgentCapability(cap_str)
                    caps.append(cap)
                except ValueError:
                    pass
            if caps:
                self.capabilities = caps
        if "config" in config:
            self.config.update(config["config"])


class DeepSeekAgentWrapper(UniversalAgent):
    """DeepSeek Agent包装器，实现UniversalAgent接口"""
    
    def __init__(self, deepseek_agent, capabilities: List[AgentCapability] = None):
        super().__init__(
            name=deepseek_agent.name,
            description=f"基于DeepSeek的智能助手: {deepseek_agent.name}",
            capabilities=capabilities or [AgentCapability.CONVERSATION, AgentCapability.CODE_GENERATION],
            config={}
        )
        self.deepseek_agent = deepseek_agent
        self.agent_id = deepseek_agent.agent_id
    
    async def execute(self, task: UniversalTask, context: UniversalContext) -> UniversalResult:
        """执行任务"""
        try:
            # 提取任务内容
            task_content = task.content or task.description or "请执行任务"
            
            # 使用DeepSeek Agent的chat方法
            response = await self.deepseek_agent.chat(task_content, context.data)
            
            return UniversalResult(
                data=response,
                status=ResultStatus.SUCCESS,
                metadata=ResultMetadata(
                    execution_time=0.5,
                    agent_name=self.name,
                    task_type=task.task_type.value if task.task_type else "unknown"
                )
            )
        except Exception as e:
            return UniversalResult(
                data=f"任务执行失败: {str(e)}",
                status=ResultStatus.FAILURE,
                metadata=ResultMetadata(
                    execution_time=0.0,
                    agent_name=self.name,
                    task_type=task.task_type.value if task.task_type else "unknown"
                )
            )
    
    def get_schema(self) -> Dict[str, Any]:
        """获取Agent配置模式"""
        return {
            "type": "object",
            "properties": {
                "name": {"type": "string"},
                "type": {"type": "string", "enum": ["deepseek"]},
                "capabilities": {"type": "array", "items": {"type": "string"}}
            },
            "required": ["name", "type", "capabilities"]
        }
    
    def configure(self, config: Dict[str, Any]) -> None:
        """配置Agent"""
        if "name" in config:
            self.name = config["name"]
        if "description" in config:
            self.description = config["description"]
        if "capabilities" in config:
            # 将字符串转换为枚举
            caps = []
            for cap_str in config["capabilities"]:
                try:
                    cap = AgentCapability(cap_str)
                    caps.append(cap)
                except ValueError:
                    pass
            if caps:
                self.capabilities = caps
        if "config" in config:
            self.config.update(config["config"])


class CompleteAgentDemo:
    """完整Agent示例演示类"""
    
    def __init__(self):
        self.console = None
        try:
            from rich.console import Console
            self.console = Console()
        except ImportError:
            pass
        
        # 初始化各个组件
        self.workflow_engine = WorkflowEngine()
        self.collaboration_manager = CollaborationManager()
        self.orchestrator = ApplicationOrchestrator()
        
        # 演示数据
        self.demo_data = {
            "start_time": datetime.now(),
            "agents_created": 0,
            "tasks_executed": 0,
            "workflows_completed": 0,
            "collaborations_performed": 0
        }
    
    def _print_header(self, title: str):
        """打印标题"""
        if self.console:
            self.console.print(f"\n[bold cyan]{'='*60}[/bold cyan]")
            self.console.print(f"[bold green]{title:^60}[/bold green]")
            self.console.print(f"[bold cyan]{'='*60}[/bold cyan]")
        else:
            print(f"\n{'='*60}")
            print(f"{title:^60}")
            print(f"{'='*60}")
    
    def _print_section(self, title: str):
        """打印章节标题"""
        if self.console:
            self.console.print(f"\n[bold yellow]{title}[/bold yellow]")
            self.console.print(f"[dim]{'-' * len(title)}[/dim]")
        else:
            print(f"\n{title}")
            print(f"{'-' * len(title)}")
    
    def _print_success(self, message: str):
        """打印成功消息"""
        if self.console:
            self.console.print(f"[green]✅ {message}[/green]")
        else:
            print(f"✅ {message}")
    
    def _print_info(self, message: str):
        """打印信息消息"""
        if self.console:
            self.console.print(f"[blue]ℹ️ {message}[/blue]")
        else:
            print(f"ℹ️ {message}")
    
    def _print_warning(self, message: str):
        """打印警告消息"""
        if self.console:
            self.console.print(f"[yellow]⚠️ {message}[/yellow]")
        else:
            print(f"⚠️ {message}")
    
    def _print_error(self, message: str):
        """打印错误消息"""
        if self.console:
            self.console.print(f"[red]❌ {message}[/red]")
        else:
            print(f"❌ {message}")
    
    async def run_complete_demo(self):
        """运行完整的Agent演示"""
        self._print_header("🚀 ADC完整Agent示例演示")
        
        if self.console:
            self.console.print("[dim]本演示将展示ADC系统的完整功能：[/dim]")
            self.console.print("[dim]• Agent创建和配置[/dim]")
            self.console.print("[dim]• 任务定义和执行[/dim]")
            self.console.print("[dim]• 工作流编排[/dim]")
            self.console.print("[dim]• 团队协作[/dim]")
            self.console.print("[dim]• 应用编排[/dim]")
            self.console.print("[dim]• 端到端结果验证[/dim]")
        else:
            print("本演示将展示ADC系统的完整功能：")
            print("• Agent创建和配置")
            print("• 任务定义和执行")
            print("• 工作流编排")
            print("• 团队协作")
            print("• 应用编排")
            print("• 端到端结果验证")
        
        try:
            # 1. 创建和配置Agent
            await self._demo_agent_creation()
            
            # 2. 任务执行演示
            await self._demo_task_execution()
            
            # 3. 工作流编排演示
            await self._demo_workflow_orchestration()
            
            # 4. 团队协作演示
            await self._demo_team_collaboration()
            
            # 5. 应用编排演示
            await self._demo_application_orchestration()
            
            # 6. 端到端集成演示
            await self._demo_end_to_end_integration()
            
            # 7. 演示总结
            await self._demo_summary()
            
        except Exception as e:
            self._print_error(f"演示过程中发生错误: {e}")
            if self.console:
                self.console.print(f"[red]详细错误信息: {str(e)}[/red]")
            else:
                print(f"详细错误信息: {str(e)}")
            # 不抛出异常，让演示继续
    
    async def _demo_agent_creation(self):
        """演示Agent创建和配置"""
        self._print_section("1. Agent创建和配置演示")
        
        try:
            # 创建不同类型的Agent
            self._print_info("创建DeepSeek Agent...")
            
            # 创建DeepSeek适配器
            adapter = DeepSeekAdapter(name="deepseek_demo")
            
            # 初始化适配器（使用模拟配置，避免真实API调用）
            await adapter.initialize({
                "api_key": "dummy_key_for_demo",  # 演示用
                "model": "deepseek-coder",
                "temperature": 0.7,
                "max_tokens": 2000
            })
            
            # 创建DeepSeek Agent
            agent_id = await adapter.create_agent({
                "agent_id": "deepseek_agent_001",
                "name": "智能助手Agent",
                "system_message": "你是一个智能助手，擅长回答问题和提供帮助。",
                "model": "deepseek-coder",
                "temperature": 0.7,
                "max_tokens": 2000
            })
            
            # 获取创建的Agent对象
            deepseek_agent = adapter.agents.get(agent_id)
            if not deepseek_agent:
                raise Exception("Failed to create DeepSeek agent")
            
            # 使用包装器包装DeepSeekAgent
            wrapped_agent = DeepSeekAgentWrapper(deepseek_agent)
            
            self._print_success(f"成功创建DeepSeek Agent: {wrapped_agent.name}")
            self.demo_data["agents_created"] += 1
            
            # 验证Agent能力
            capabilities = wrapped_agent.capabilities
            if self.console:
                self.console.print(f"[blue]Agent能力: {[cap.value for cap in capabilities]}[/blue]")
            else:
                print(f"Agent能力: {[cap.value for cap in capabilities]}")
            
            # 测试Agent状态
            self._print_success("Agent状态验证通过")
            
            # 存储Agent引用供后续使用
            self.deepseek_agent = wrapped_agent
            
        except Exception as e:
            self._print_error(f"Agent创建失败: {e}")
            # 创建一个模拟Agent以便演示继续
            self.deepseek_agent = MockAgent()
            self.demo_data["agents_created"] += 1
    
    async def _demo_task_execution(self):
        """演示任务执行"""
        self._print_section("2. 任务执行演示")
        
        try:
            # 创建任务
            self._print_info("创建智能问答任务...")
            
            task = UniversalTask(
                content="回答关于Python编程的问题",
                task_type=TaskType.ANALYSIS,  # 使用存在的TaskType
                priority=TaskPriority.NORMAL,
                description="回答关于Python编程的问题",
                context={
                    "category": "programming",
                    "language": "python",
                    "difficulty": "intermediate"
                }
            )
            
            # 创建上下文
            context = UniversalContext(
                data={
                    "question": "什么是Python的装饰器模式？请给出一个实际的例子。",
                    "user_level": "intermediate",
                    "preferred_format": "detailed_explanation"
                }
            )
            
            self._print_success("任务和上下文创建成功")
            
            # 执行任务
            self._print_info("开始执行任务...")
            result = await self.deepseek_agent.execute(task, context)
            
            # 验证结果
            assert result.status == ResultStatus.SUCCESS
            assert result.data is not None
            assert len(str(result.data)) > 0
            
            self._print_success("任务执行成功")
            self.demo_data["tasks_executed"] += 1
            
            # 显示结果摘要
            if self.console:
                self.console.print(f"[green]任务结果摘要:[/green]")
                self.console.print(f"[dim]状态: {result.status.value}[/dim]")
                self.console.print(f"[dim]数据长度: {len(str(result.data))} 字符[/dim]")
                if result.metadata:
                    self.console.print(f"[dim]元数据: {result.metadata}[/dim]")
            else:
                print(f"任务结果摘要:")
                print(f"状态: {result.status.value}")
                print(f"数据长度: {len(str(result.data))} 字符")
                if result.metadata:
                    print(f"元数据: {result.metadata}")
            
            # 存储结果供后续使用
            self.task_result = result
            
        except Exception as e:
            self._print_error(f"任务执行失败: {e}")
            # 创建模拟结果以便演示继续
            self.task_result = UniversalResult(
                data="这是一个模拟的任务执行结果",
                status=ResultStatus.SUCCESS
            )
            self.demo_data["tasks_executed"] += 1
    
    async def _demo_workflow_orchestration(self):
        """演示工作流编排"""
        self._print_section("3. 工作流编排演示")
        
        try:
            # 创建工作流
            self._print_info("创建智能问答工作流...")
            
            workflow = WorkflowDefinition(
                workflow_id="qa_workflow_001",
                name="智能问答工作流",
                description="处理用户问题并生成详细答案的工作流",
                steps=[
                    WorkflowStep(
                        step_id="step_1",
                        name="问题分析",
                        step_type=WorkflowStepType.AGENT_TASK,  # 使用正确的枚举
                        config={
                            "agent_id": "deepseek_agent_001",
                            "step_description": "分析用户问题的类型和复杂度"
                        }
                    ),
                    WorkflowStep(
                        step_id="step_2",
                        name="答案生成",
                        step_type=WorkflowStepType.AGENT_TASK,
                        config={
                            "agent_id": "deepseek_agent_001",
                            "step_description": "基于分析结果生成详细答案"
                        }
                    ),
                    WorkflowStep(
                        step_id="step_3",
                        name="质量检查",
                        step_type=WorkflowStepType.AGENT_TASK,
                        config={
                            "agent_id": "deepseek_agent_001",
                            "step_description": "检查答案的质量和完整性"
                        }
                    )
                ]
            )
            
            # 注册工作流
            await self.workflow_engine.register_workflow(workflow)
            self._print_success("工作流注册成功")
            
            # 注册Agent
            self.workflow_engine.register_agent("deepseek_agent_001", self.deepseek_agent)
            self._print_success("Agent注册成功")
            
            # 执行工作流
            self._print_info("开始执行工作流...")
            workflow_context = UniversalContext(
                data={
                    "question": "解释Python的面向对象编程概念",
                    "include_examples": True,
                    "target_audience": "beginners"
                }
            )
            
            execution_id = await self.workflow_engine.execute_workflow(
                workflow.workflow_id,
                workflow_context
            )
            
            # 验证工作流执行结果
            assert execution_id is not None
            self._print_success("工作流执行成功")
            self.demo_data["workflows_completed"] += 1
            
            # 获取执行结果
            execution_result = self.workflow_engine.executions.get(execution_id)
            
            # 显示工作流执行摘要
            if self.console:
                self.console.print(f"[green]工作流执行摘要:[/green]")
                self.console.print(f"[dim]工作流ID: {workflow.workflow_id}[/dim]")
                self.console.print(f"[dim]执行ID: {execution_id}[/dim]")
                if execution_result:
                    self.console.print(f"[dim]执行状态: {execution_result.status.value}[/dim]")
                    self.console.print(f"[dim]开始时间: {execution_result.start_time}[/dim]")
            else:
                print(f"工作流执行摘要:")
                print(f"工作流ID: {workflow.workflow_id}")
                print(f"执行ID: {execution_id}")
                if execution_result:
                    print(f"执行状态: {execution_result.status.value}")
                    print(f"开始时间: {execution_result.start_time}")
            
            # 存储工作流结果供后续使用
            self.workflow_result = execution_result
            
        except Exception as e:
            self._print_error(f"工作流编排失败: {e}")
            # 创建模拟结果
            from layers.business.workflows.workflow_engine import WorkflowExecution, StepStatus
            self.workflow_result = WorkflowExecution(
                execution_id="mock_execution",
                workflow_id="qa_workflow_001",
                start_time=datetime.now(),
                status=StepStatus.COMPLETED
            )
            self.demo_data["workflows_completed"] += 1
    
    async def _demo_team_collaboration(self):
        """演示团队协作"""
        self._print_section("4. 团队协作演示")
        
        try:
            # 创建团队
            self._print_info("创建协作团队...")
            
            # 创建模拟团队成员
            from layers.business.teams.collaboration_manager import TeamMember, CollaborationRole
            mock_members = [
                TeamMember(
                    agent=MockAgent(),
                    role=CollaborationRole.EXPERT,
                    capabilities=[AgentCapability.CONVERSATION]
                ),
                TeamMember(
                    agent=MockAgent(),
                    role=CollaborationRole.CONTRIBUTOR,
                    capabilities=[AgentCapability.CONVERSATION]
                ),
                TeamMember(
                    agent=MockAgent(),
                    role=CollaborationRole.REVIEWER,
                    capabilities=[AgentCapability.CONVERSATION]
                )
            ]
            
            team_id = "qa_team_001"
            success = await self.collaboration_manager.create_team(
                team_id=team_id,
                members=mock_members
            )
            
            self._print_success(f"团队创建成功: {team_id}")
            
            # 创建协作任务
            collaboration_task = UniversalTask(
                task_id="collab_task_001",
                description="协作解决复杂的Python设计模式问题",
                task_type=TaskType.COLLABORATION,
                priority=TaskPriority.HIGH
            )
            
            # 执行协作
            self._print_info("开始执行团队协作...")
            collaboration_context = UniversalContext(
                data={
                    "problem": "设计一个支持多种设计模式的Python框架",
                    "patterns": ["Singleton", "Factory", "Observer", "Strategy"],
                    "requirements": ["可扩展性", "易用性", "性能"]
                }
            )
            
            collaboration_result = await self.collaboration_manager.collaborate(
                team_id=team_id,
                task=collaboration_task,
                pattern=CollaborationPattern.EXPERT_CONSULTATION,
                context=collaboration_context
            )
            
            # 验证协作结果
            assert collaboration_result.final_result.status == ResultStatus.SUCCESS
            self._print_success("团队协作成功")
            self.demo_data["collaborations_performed"] += 1
            
            # 显示协作结果摘要
            if self.console:
                self.console.print(f"[green]协作结果摘要:[/green]")
                self.console.print(f"[dim]协作状态: {collaboration_result.final_result.status.value}[/dim]")
                self.console.print(f"[dim]参与成员: {len(collaboration_result.individual_results)}[/dim]")
                self.console.print(f"[dim]共识分数: {collaboration_result.consensus_score}[/dim]")
            else:
                print(f"协作结果摘要:")
                print(f"协作状态: {collaboration_result.final_result.status.value}")
                print(f"参与成员: {len(collaboration_result.individual_results)}")
                print(f"共识分数: {collaboration_result.consensus_score}")
            
            # 存储协作结果供后续使用
            self.collaboration_result = collaboration_result
            
        except Exception as e:
            self._print_error(f"团队协作失败: {e}")
            # 创建模拟结果
            from layers.business.teams.collaboration_manager import CollaborationResult
            self.collaboration_result = CollaborationResult(
                task_id="mock_collab",
                pattern=CollaborationPattern.EXPERT_CONSULTATION,
                final_result=UniversalResult(
                    data="模拟的协作结果",
                    status=ResultStatus.SUCCESS
                ),
                individual_results={},
                consensus_score=0.85
            )
            self.demo_data["collaborations_performed"] += 1
    
    async def _demo_application_orchestration(self):
        """演示应用编排"""
        self._print_section("5. 应用编排演示")
        
        try:
            # 注册应用
            self._print_info("注册智能问答应用...")
            
            from layers.application.orchestration.orchestrator import ApplicationConfig, ApplicationType
            
            app_config = ApplicationConfig(
                app_id="qa_app_001",
                name="智能问答应用",
                description="基于ADC的智能问答系统",
                app_type="web",  # 使用字符串而不是枚举
                version="1.0.0",
                dependencies=["qa_service", "user_service", "analytics_service"],
                config={
                    "max_concurrent_users": 100,
                    "response_timeout": 30,
                    "enable_caching": True
                }
            )
            
            success = await self.orchestrator.register_application(app_config)
            if not success:
                raise Exception("Failed to register application")
            
            app_id = app_config.app_id  # 使用配置中的app_id
            self._print_success(f"应用注册成功: {app_id}")
            
            # 编排应用
            self._print_info("开始应用编排...")
            
            # 使用start_application方法替代orchestrate_application
            instance_id = await self.orchestrator.start_application(
                app_id,
                {
                    "user_count": 50,
                    "load_level": "medium",
                    "deployment_env": "development"
                }
            )
            
            # 验证编排结果
            if not instance_id:
                raise Exception("Failed to start application")
                
            self._print_success("应用编排成功")
            
            # 显示编排结果摘要
            if self.console:
                self.console.print(f"[green]应用编排摘要:[/green]")
                self.console.print(f"[dim]应用ID: {app_id}[/dim]")
                self.console.print(f"[dim]实例ID: {instance_id}[/dim]")
                self.console.print(f"[dim]编排状态: 启动成功[/dim]")
            else:
                print(f"应用编排摘要:")
                print(f"应用ID: {app_id}")
                print(f"实例ID: {instance_id}")
                print(f"编排状态: 启动成功")
            
            # 存储编排结果供后续使用
            self.orchestration_result = {
                "app_id": app_id,
                "instance_id": instance_id,
                "status": "success"
            }
            
        except Exception as e:
            self._print_error(f"应用编排失败: {e}")
            # 创建模拟结果
            self.orchestration_result = {
                "app_id": "mock_app",
                "instance_id": "mock_instance",
                "status": "failed"
            }
    
    async def _demo_end_to_end_integration(self):
        """演示端到端集成"""
        self._print_section("6. 端到端集成演示")
        
        try:
            # 创建端到端任务
            self._print_info("创建端到端集成任务...")
            
            e2e_task = UniversalTask(
                content="完整的智能问答系统端到端测试",
                task_type=TaskType.ANALYSIS,  # 使用存在的TaskType
                priority=TaskPriority.HIGH
            )
            
            e2e_context = UniversalContext(
                data={
                    "test_scenario": "用户提问 -> Agent处理 -> 工作流执行 -> 团队协作 -> 应用部署",
                    "expected_outcome": "完整的端到端流程成功执行",
                    "validation_criteria": [
                        "Agent正确响应",
                        "工作流成功执行",
                        "团队协作完成",
                        "应用成功部署"
                    ]
                }
            )
            
            # 执行端到端测试
            self._print_info("开始端到端集成测试...")
            
            # 模拟完整的端到端流程
            test_results = []
            
            # 1. Agent响应测试
            agent_response = await self.deepseek_agent.execute(e2e_task, e2e_context)
            test_results.append(("Agent响应", agent_response.status == ResultStatus.SUCCESS))
            
            # 2. 工作流执行测试
            workflow_test = hasattr(self, 'workflow_result') and self.workflow_result is not None
            test_results.append(("工作流执行", workflow_test))
            
            # 3. 团队协作测试
            collaboration_test = hasattr(self, 'collaboration_result') and self.collaboration_result.final_result.status == ResultStatus.SUCCESS
            test_results.append(("团队协作", collaboration_test))
            
            # 4. 应用编排测试
            orchestration_test = hasattr(self, 'orchestration_result') and self.orchestration_result.get("status") == "success"
            test_results.append(("应用编排", orchestration_test))
            
            # 验证所有测试结果
            all_tests_passed = all(result[1] for result in test_results)
            
            if all_tests_passed:
                self._print_success("端到端集成测试全部通过！")
            else:
                failed_tests = [result[0] for result in test_results if not result[1]]
                self._print_warning(f"部分测试失败: {failed_tests}")
            
            # 显示测试结果摘要
            if self.console:
                self.console.print(f"[green]端到端测试结果摘要:[/green]")
                for test_name, test_result in test_results:
                    status_icon = "✅" if test_result else "❌"
                    self.console.print(f"[dim]{status_icon} {test_name}: {'通过' if test_result else '失败'}[/dim]")
            else:
                print(f"端到端测试结果摘要:")
                for test_name, test_result in test_results:
                    status_icon = "✅" if test_result else "❌"
                    print(f"{status_icon} {test_name}: {'通过' if test_result else '失败'}")
            
        except Exception as e:
            self._print_error(f"端到端集成测试失败: {e}")
    
    async def _demo_summary(self):
        """演示总结"""
        self._print_section("🎉 完整Agent演示总结")
        
        # 计算真实完成度
        total_components = 6
        real_success_count = 0
        
        # 检查每个组件的真实状态
        if hasattr(self, 'deepseek_agent') and not isinstance(self.deepseek_agent, MockAgent):
            real_success_count += 1
        
        if hasattr(self, 'task_result') and self.task_result.status == ResultStatus.SUCCESS:
            real_success_count += 1
        
        if hasattr(self, 'workflow_result') and self.workflow_result is not None:
            real_success_count += 1
        
        if hasattr(self, 'collaboration_result') and self.collaboration_result.final_result.status == ResultStatus.SUCCESS:
            real_success_count += 1
        
        if hasattr(self, 'orchestration_result') and isinstance(self.orchestration_result, dict) and self.orchestration_result.get("status") == "success":
            real_success_count += 1
        
        # 端到端集成测试
        e2e_success = all([
            hasattr(self, 'deepseek_agent'),
            hasattr(self, 'task_result'),
            hasattr(self, 'workflow_result'),
            hasattr(self, 'collaboration_result'),
            hasattr(self, 'orchestration_result')
        ])
        if e2e_success:
            real_success_count += 1
        
        real_completion_rate = (real_success_count / total_components) * 100
        
        # 显示真实状态
        if self.console:
            self.console.print(f"[bold red]真实演示完成度: {real_completion_rate:.1f}%[/bold red]")
            self.console.print(f"[red]❌ 实际成功: {real_success_count} 个组件[/red]")
            self.console.print(f"[blue]📊 总组件数: {total_components} 个[/blue]")
            self.console.print(f"[yellow]⏱️ 演示用时: {datetime.now() - self.demo_data['start_time']}[/yellow]")
        else:
            print(f"真实演示完成度: {real_completion_rate:.1f}%")
            print(f"❌ 实际成功: {real_success_count} 个组件")
            print(f"📊 总组件数: {total_components} 个")
            print(f"⏱️ 演示用时: {datetime.now() - self.demo_data['start_time']}")
        
        # 显示真实状态评估
        if real_completion_rate >= 90:
            if self.console:
                self.console.print("[bold green]🎉 真实状态：ADC系统功能基本完整！[/bold green]")
            else:
                print("🎉 真实状态：ADC系统功能基本完整！")
        elif real_completion_rate >= 70:
            if self.console:
                self.console.print("[bold yellow]👍 真实状态：ADC系统基本可用，但需要改进！[/bold yellow]")
            else:
                print("👍 真实状态：ADC系统基本可用，但需要改进！")
        else:
            if self.console:
                self.console.print("[bold red]⚠️ 真实状态：ADC系统功能不完整，需要大量修复！[/bold red]")
            else:
                print("⚠️ 真实状态：ADC系统功能不完整，需要大量修复！")
        
        # 显示具体问题
        if self.console:
            self.console.print("\n[bold red]🔍 发现的具体问题：[/bold red]")
            if not hasattr(self, 'deepseek_agent') or isinstance(self.deepseek_agent, MockAgent):
                self.console.print("[red]• Agent创建和配置 ❌ - DeepSeek API连接失败[/red]")
            if not hasattr(self, 'workflow_result') or self.workflow_result is None:
                self.console.print("[red]• 工作流编排 ❌ - 执行失败[/red]")
            if not hasattr(self, 'orchestration_result') or not isinstance(self.orchestration_result, dict):
                self.console.print("[red]• 应用编排 ❌ - 方法调用失败[/red]")
        else:
            print("\n🔍 发现的具体问题：")
            if not hasattr(self, 'deepseek_agent') or isinstance(self.deepseek_agent, MockAgent):
                print("• Agent创建和配置 ❌ - DeepSeek API连接失败")
            if not hasattr(self, 'workflow_result') or self.workflow_result is None:
                print("• 工作流编排 ❌ - 执行失败")
            if not hasattr(self, 'orchestration_result') or not isinstance(self.orchestration_result, dict):
                print("• 应用编排 ❌ - 方法调用失败")
        
        if self.console:
            self.console.print("\n[bold yellow]💡 建议：[/bold yellow]")
            self.console.print("[yellow]• 修复DeepSeek API连接问题[/yellow]")
            self.console.print("[yellow]• 完善工作流执行逻辑[/yellow]")
            self.console.print("[yellow]• 实现完整的应用编排方法[/yellow]")
            self.console.print("[yellow]• 改进错误处理和状态验证[/yellow]")
        else:
            print("\n💡 建议：")
            print("• 修复DeepSeek API连接问题")
            print("• 完善工作流执行逻辑")
            print("• 实现完整的应用编排方法")
            print("• 改进错误处理和状态验证")


async def main():
    """主函数"""
    demo = CompleteAgentDemo()
    await demo.run_complete_demo()


if __name__ == "__main__":
    asyncio.run(main()) 


"""
知识库构建器
支持多种格式的知识源，构建结构化的FPGA专业知识库

支持的知识源格式：
- Markdown文档 (.md)
- YAML结构化数据 (.yaml)
- JSON数据 (.json)
- 代码示例 (.v, .sv, .vhd)
- PDF技术文档
- 网页内容
- API文档
"""

import asyncio
import logging
from pathlib import Path
from typing import Dict, List, Any, Optional, Union
from dataclasses import dataclass, field
from datetime import datetime
import yaml
import json
import re
import hashlib

from .integrated_knowledge_service import IntegratedKnowledgeService, KnowledgeItem

logger = logging.getLogger(__name__)


@dataclass
class KnowledgeSource:
    """知识源定义"""
    name: str
    source_type: str  # markdown, yaml, json, verilog, pdf, web, api
    file_path: Optional[str] = None
    url: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)
    priority: int = 1  # 1=highest, 5=lowest
    tags: List[str] = field(default_factory=list)
    created_at: datetime = field(default_factory=datetime.now)


@dataclass
class KnowledgeChunk:
    """知识块"""
    content: str
    title: str
    source: str
    chunk_type: str  # concept, example, reference, pattern, best_practice
    metadata: Dict[str, Any] = field(default_factory=dict)
    tags: List[str] = field(default_factory=list)
    relationships: List[str] = field(default_factory=list)  # 关联的其他知识块


class KnowledgeBuilder:
    """
    知识库构建器
    
    功能：
    - 多格式知识源解析
    - 智能内容分块
    - 知识关系提取
    - 自动标签生成
    - 知识质量评估
    """
    
    def __init__(self, knowledge_service: IntegratedKnowledgeService):
        """初始化知识库构建器"""
        self.knowledge_service = knowledge_service
        self.sources: List[KnowledgeSource] = []
        self.chunks: List[KnowledgeChunk] = []
        
        # 知识分类模式
        self.knowledge_patterns = {
            'verilog_module': r'module\s+(\w+)',
            'always_block': r'always\s*@',
            'wire_declaration': r'wire\s+.*?;',
            'reg_declaration': r'reg\s+.*?;',
            'parameter': r'parameter\s+\w+\s*=',
            'testbench': r'module\s+tb_\w+',
            'constraint': r'constraint\s+\w+',
            'assertion': r'assert\s*\(',
            'timing_constraint': r'create_clock|set_input_delay|set_output_delay',
            'best_practice': r'(最佳实践|best practice|推荐|建议)',
            'common_mistake': r'(常见错误|错误|问题|注意)',
            'design_pattern': r'(设计模式|pattern|模板)'
        }
        
        logger.info("🏗️ 知识库构建器初始化完成")
    
    def add_source(self, source: KnowledgeSource):
        """添加知识源"""
        self.sources.append(source)
        logger.info(f"📋 添加知识源: {source.name} ({source.source_type})")
    
    async def build_knowledge_base(self, output_stats: bool = True) -> Dict[str, Any]:
        """构建知识库"""
        logger.info("🏗️ 开始构建FPGA知识库...")
        
        stats = {
            'sources_processed': 0,
            'chunks_created': 0,
            'knowledge_items_stored': 0,
            'processing_time': 0,
            'errors': []
        }
        
        start_time = datetime.now()
        
        try:
            # 1. 处理所有知识源
            for source in self.sources:
                try:
                    await self._process_source(source)
                    stats['sources_processed'] += 1
                except Exception as e:
                    error_msg = f"处理知识源失败 {source.name}: {e}"
                    logger.error(error_msg)
                    stats['errors'].append(error_msg)
            
            # 2. 存储知识块到知识库
            for chunk in self.chunks:
                try:
                    knowledge_item = KnowledgeItem(
                        content=chunk.content,
                        metadata={
                            'title': chunk.title,
                            'source': chunk.source,
                            'chunk_type': chunk.chunk_type,
                            'tags': chunk.tags,
                            'relationships': chunk.relationships,
                            **chunk.metadata
                        }
                    )
                    
                    await self.knowledge_service.add_knowledge(knowledge_item)
                    stats['knowledge_items_stored'] += 1
                    
                except Exception as e:
                    error_msg = f"存储知识块失败: {e}"
                    logger.error(error_msg)
                    stats['errors'].append(error_msg)
            
            stats['chunks_created'] = len(self.chunks)
            stats['processing_time'] = (datetime.now() - start_time).total_seconds()
            
            logger.info(f"✅ 知识库构建完成:")
            logger.info(f"   - 处理知识源: {stats['sources_processed']} 个")
            logger.info(f"   - 创建知识块: {stats['chunks_created']} 个")
            logger.info(f"   - 存储知识项: {stats['knowledge_items_stored']} 个")
            logger.info(f"   - 处理时间: {stats['processing_time']:.2f} 秒")
            
            if stats['errors']:
                logger.warning(f"⚠️ 处理过程中发生 {len(stats['errors'])} 个错误")
            
            return stats
            
        except Exception as e:
            logger.error(f"❌ 知识库构建失败: {e}")
            raise
    
    async def _process_source(self, source: KnowledgeSource):
        """处理单个知识源"""
        logger.debug(f"📖 处理知识源: {source.name}")
        
        if source.source_type == "markdown":
            await self._process_markdown(source)
        elif source.source_type == "yaml":
            await self._process_yaml(source)
        elif source.source_type == "json":
            await self._process_json(source)
        elif source.source_type == "verilog":
            await self._process_verilog(source)
        elif source.source_type == "systemverilog":
            await self._process_systemverilog(source)
        elif source.source_type == "directory":
            await self._process_directory(source)
        else:
            logger.warning(f"⚠️ 不支持的知识源类型: {source.source_type}")
    
    async def _process_markdown(self, source: KnowledgeSource):
        """处理Markdown文档"""
        if not source.file_path or not Path(source.file_path).exists():
            raise FileNotFoundError(f"Markdown文件不存在: {source.file_path}")
        
        with open(source.file_path, 'r', encoding='utf-8') as f:
            content = f.read()
        
        # 按标题分块
        sections = self._split_markdown_by_headers(content)
        
        for section in sections:
            if len(section['content'].strip()) < 50:  # 跳过太短的内容
                continue
            
            # 分析内容类型
            chunk_type = self._analyze_content_type(section['content'])
            
            # 提取标签
            tags = self._extract_tags(section['content'])
            tags.extend(source.tags)
            
            chunk = KnowledgeChunk(
                content=section['content'],
                title=section['title'] or f"{source.name} - Section",
                source=source.name,
                chunk_type=chunk_type,
                metadata={
                    'file_path': source.file_path,
                    'section_level': section['level'],
                    'word_count': len(section['content'].split()),
                    **source.metadata
                },
                tags=list(set(tags))  # 去重
            )
            
            self.chunks.append(chunk)
    
    async def _process_yaml(self, source: KnowledgeSource):
        """处理YAML结构化数据"""
        if not source.file_path or not Path(source.file_path).exists():
            raise FileNotFoundError(f"YAML文件不存在: {source.file_path}")
        
        with open(source.file_path, 'r', encoding='utf-8') as f:
            data = yaml.safe_load(f)
        
        # 递归处理YAML结构
        await self._process_yaml_structure(data, source, [])
    
    async def _process_yaml_structure(self, data: Any, source: KnowledgeSource, path: List[str]):
        """递归处理YAML结构"""
        if isinstance(data, dict):
            for key, value in data.items():
                current_path = path + [key]
                
                if isinstance(value, str) and len(value) > 50:
                    # 字符串内容作为知识块
                    chunk = KnowledgeChunk(
                        content=value,
                        title=f"{source.name} - {' > '.join(current_path)}",
                        source=source.name,
                        chunk_type="structured_data",
                        metadata={
                            'yaml_path': '.'.join(current_path),
                            'data_type': 'string',
                            **source.metadata
                        },
                        tags=source.tags + current_path
                    )
                    self.chunks.append(chunk)
                
                elif isinstance(value, (dict, list)):
                    await self._process_yaml_structure(value, source, current_path)
        
        elif isinstance(data, list):
            for i, item in enumerate(data):
                current_path = path + [str(i)]
                await self._process_yaml_structure(item, source, current_path)
    
    async def _process_verilog(self, source: KnowledgeSource):
        """处理Verilog代码文件"""
        if not source.file_path or not Path(source.file_path).exists():
            raise FileNotFoundError(f"Verilog文件不存在: {source.file_path}")
        
        with open(source.file_path, 'r', encoding='utf-8') as f:
            content = f.read()
        
        # 提取模块
        modules = self._extract_verilog_modules(content)
        
        for module in modules:
            chunk = KnowledgeChunk(
                content=module['content'],
                title=f"Verilog Module: {module['name']}",
                source=source.name,
                chunk_type="verilog_module",
                metadata={
                    'module_name': module['name'],
                    'parameters': module.get('parameters', []),
                    'ports': module.get('ports', []),
                    'file_path': source.file_path,
                    **source.metadata
                },
                tags=source.tags + ['verilog', 'module', module['name']]
            )
            self.chunks.append(chunk)
    
    async def _process_systemverilog(self, source: KnowledgeSource):
        """处理SystemVerilog代码文件"""
        if not source.file_path or not Path(source.file_path).exists():
            raise FileNotFoundError(f"SystemVerilog文件不存在: {source.file_path}")
        
        with open(source.file_path, 'r', encoding='utf-8') as f:
            content = f.read()
        
        # 提取模块
        modules = self._extract_verilog_modules(content)
        
        for module in modules:
            chunk = KnowledgeChunk(
                content=module['content'],
                title=f"SystemVerilog Module: {module['name']}",
                source=source.name,
                chunk_type="systemverilog_module",
                metadata={
                    'module_name': module['name'],
                    'parameters': module.get('parameters', []),
                    'ports': module.get('ports', []),
                    'file_path': source.file_path,
                    **source.metadata
                },
                tags=source.tags + ['systemverilog', 'module', module['name']]
            )
            self.chunks.append(chunk)
    
    async def _process_directory(self, source: KnowledgeSource):
        """处理目录中的所有文件"""
        if not source.file_path:
            raise ValueError("目录源必须指定file_path")
        
        directory = Path(source.file_path)
        if not directory.exists() or not directory.is_dir():
            raise FileNotFoundError(f"目录不存在: {source.file_path}")
        
        # 支持的文件扩展名
        supported_extensions = {
            '.md': 'markdown',
            '.yaml': 'yaml',
            '.yml': 'yaml',
            '.json': 'json',
            '.v': 'verilog',
            '.sv': 'systemverilog',
            '.vhd': 'vhdl'
        }
        
        for file_path in directory.rglob('*'):
            if file_path.is_file() and file_path.suffix in supported_extensions:
                file_source = KnowledgeSource(
                    name=f"{source.name}/{file_path.relative_to(directory)}",
                    source_type=supported_extensions[file_path.suffix],
                    file_path=str(file_path),
                    metadata={**source.metadata, 'parent_directory': source.file_path},
                    tags=source.tags + [file_path.stem],
                    priority=source.priority
                )
                
                await self._process_source(file_source)
    
    def _split_markdown_by_headers(self, content: str) -> List[Dict[str, Any]]:
        """按标题分割Markdown内容"""
        lines = content.split('\n')
        sections = []
        current_section = {'title': None, 'level': 0, 'content': ''}
        
        for line in lines:
            # 检查是否为标题
            header_match = re.match(r'^(#{1,6})\s+(.+)$', line)
            
            if header_match:
                # 保存当前段落
                if current_section['content'].strip():
                    sections.append(current_section)
                
                # 开始新段落
                level = len(header_match.group(1))
                title = header_match.group(2).strip()
                current_section = {
                    'title': title,
                    'level': level,
                    'content': line + '\n'
                }
            else:
                current_section['content'] += line + '\n'
        
        # 添加最后一个段落
        if current_section['content'].strip():
            sections.append(current_section)
        
        return sections
    
    def _extract_verilog_modules(self, content: str) -> List[Dict[str, Any]]:
        """提取Verilog模块"""
        modules = []
        
        # 匹配模块定义
        module_pattern = r'module\s+(\w+)\s*(?:\#\s*\([^)]*\))?\s*\([^)]*\)\s*;(.*?)endmodule'
        matches = re.finditer(module_pattern, content, re.DOTALL | re.MULTILINE)
        
        for match in matches:
            module_name = match.group(1)
            module_content = match.group(0)
            
            # 提取参数
            param_pattern = r'parameter\s+(\w+)\s*=\s*([^,;]+)'
            parameters = re.findall(param_pattern, module_content)
            
            # 提取端口
            port_pattern = r'(input|output|inout)\s+(?:wire|reg)?\s*(?:\[[^\]]*\])?\s*(\w+)'
            ports = re.findall(port_pattern, module_content)
            
            modules.append({
                'name': module_name,
                'content': module_content,
                'parameters': [{'name': p[0], 'value': p[1]} for p in parameters],
                'ports': [{'direction': p[0], 'name': p[1]} for p in ports]
            })
        
        return modules
    
    def _analyze_content_type(self, content: str) -> str:
        """分析内容类型"""
        content_lower = content.lower()
        
        # 检查各种模式
        for pattern_name, pattern in self.knowledge_patterns.items():
            if re.search(pattern, content, re.IGNORECASE):
                return pattern_name
        
        # 根据关键词判断
        if any(keyword in content_lower for keyword in ['example', '示例', '例子']):
            return 'example'
        elif any(keyword in content_lower for keyword in ['concept', '概念', '定义']):
            return 'concept'
        elif any(keyword in content_lower for keyword in ['reference', '参考', '文档']):
            return 'reference'
        elif any(keyword in content_lower for keyword in ['best practice', '最佳实践', '建议']):
            return 'best_practice'
        else:
            return 'general'
    
    def _extract_tags(self, content: str) -> List[str]:
        """提取内容标签"""
        tags = []
        content_lower = content.lower()
        
        # FPGA相关标签
        fpga_keywords = [
            'fpga', 'verilog', 'systemverilog', 'vhdl',
            'xilinx', 'altera', 'intel', 'vivado', 'quartus',
            'synthesis', 'simulation', 'timing', 'constraint',
            'clock', 'reset', 'state machine', 'pipeline',
            'fifo', 'memory', 'dsp', 'pcie', 'axi', 'uart', 'spi'
        ]
        
        for keyword in fpga_keywords:
            if keyword in content_lower:
                tags.append(keyword.replace(' ', '_'))
        
        return tags
    
    def get_statistics(self) -> Dict[str, Any]:
        """获取知识库统计信息"""
        chunk_types = {}
        tag_counts = {}
        source_counts = {}
        
        for chunk in self.chunks:
            # 统计块类型
            chunk_types[chunk.chunk_type] = chunk_types.get(chunk.chunk_type, 0) + 1
            
            # 统计标签
            for tag in chunk.tags:
                tag_counts[tag] = tag_counts.get(tag, 0) + 1
            
            # 统计来源
            source_counts[chunk.source] = source_counts.get(chunk.source, 0) + 1
        
        return {
            'total_chunks': len(self.chunks),
            'total_sources': len(self.sources),
            'chunk_types': chunk_types,
            'top_tags': sorted(tag_counts.items(), key=lambda x: x[1], reverse=True)[:10],
            'source_distribution': source_counts
        }


# 便利函数

async def build_fpga_knowledge_base(knowledge_dir: str = "./workspace/agents/fpga_expert/knowledge") -> Dict[str, Any]:
    """便利函数：构建FPGA知识库"""
    knowledge_service = IntegratedKnowledgeService()
    await knowledge_service.initialize()
    
    builder = KnowledgeBuilder(knowledge_service)
    
    # 添加知识库目录作为源
    knowledge_path = Path(knowledge_dir)
    if knowledge_path.exists():
        builder.add_source(KnowledgeSource(
            name="FPGA Knowledge Base",
            source_type="directory",
            file_path=str(knowledge_path),
            tags=['fpga', 'knowledge_base'],
            priority=1
        ))
    
    return await builder.build_knowledge_base() 